!     author : h. spiering
!              institut f. anorg. u. anal. chemie
!              johannes gutenberg-universitaet
!              d-55099 mainz
!              last version october 2015
      subroutine fr_mossbauer(cmch,iss,info)
     use glbconst, only: maxfr1,maxfr2,maxn,maxsub,maxss,maxthss,maxs,&
                         icstrf,ivstrf,nwr,ncl
     use comdat, only: b_data,b_theory,mxpt,mfpt,mdfpt,mvpt,mgpt,mdgpt,&
                       ichanks,name_data,name_theory
                       ! mfpt,mdfpt are transferred to the routine fit
     use complt, only: n_subset
     use comthe, only: array_fr,theoryname,thfrcase,ifrcase,ifrsto,mGa,  &
                       ibmisfit,thsum
     use comstr, only: nu_device
     implicit none

     character(len=4), intent(inout) :: cmch
     integer,          intent(in) :: iss
     integer,          intent(out):: info
     integer, parameter:: nl=18

     integer       mstruct1,istruct1,istruct2,                        &
                   l1_get,l2_get,l3_get,l1_put,l2_put,l3_put,         &
                   ipTHS,ipTH,ipTHyy,ipSI,iposLG,ipt,iaddr
     character(len=4) :: c1_prp,c2_prp,c3_prp
     character(len=64):: frthname
     common/struct/mstruct1,istruct1(0:maxfr1),istruct2(0:maxfr2),     &
                   l1_get(0:maxfr1),l2_get(maxfr2),l3_get(maxthss),    &
                   l1_put(0:maxfr1),l2_put(maxfr2),l3_put(maxthss),    &
                   c1_prp(0:maxfr1),c2_prp(maxfr2),c3_prp(maxthss),    &
                   frthname(2),                                        &
                   ipt,iposLG,ipTHS,ipTH,ipTHyy,ipSI,iaddr(2)

     real          srw8
     integer       jpolsource,jpolabsorber,km,ifrth,  &
                   iflag_basis,idum1,nLG,nAS,nSC,idum2,ak8,sk8,ien
     common/strLOC/jpolsource,jpolabsorber,km,ifrth(9),iflag_basis,&
                   idum1,nLG,nAS,nSC,idum2,ak8,sk8,srw8,ien(11)

     real          acsp,uthick_w,dum,flag_basis,dum1,channels,vm_SC,   &
                   convol,th_dum,drive_mode,channels_vmax,c_nonlinear, &
                   om_samp,sc2aperture,abmue,Dsource,ring1to8,scmue,   &
                   Daperture,Dabsorber,sc2absorber,A_aperture,         &
                   B_aperture,Dtube,alpha,dumcs
     common/acstrf/acsp,uthick_w,dum,flag_basis,dum1,channels,vm_SC,   &
                   convol,th_dum,drive_mode,channels_vmax,c_nonlinear, &
                   om_samp,sc2aperture,abmue,Dsource,ring1to8,scmue,   &
                   Daperture,Dabsorber,sc2absorber,A_aperture,         &
                   B_aperture,Dtube,alpha,dumcs(icstrf-25)

     real          avsp,f_factor,base,bg_fraction,v_max,               &
                   channel_v0,geo,vnl,                                 &
                   dumv,ctauN,ctauP,dumvs
     common/avstrf/avsp,f_factor,base,bg_fraction,v_max,               &
                   channel_v0,geo,vnl(nl),                             &
                   dumv,ctauN,ctauP,dumvs(ivstrf-10-nl)

      integer    kshandle,hold,whandle,idum
      common/win/kshandle,hold,whandle(maxs),idum(18*maxs+404)

     character(len=132):: ch132,cp132
     character(len=1)  :: ch1
     character(len=40),dimension(3):: text

     integer lstruct1,lstruct2,lstruct3,                               &
             nchanDA,nchanLG,nchanTHS,nchanTH,nchansp,                 &
             mi,k1,k2,isub,ak2,sk2,kzl,kzla,kzls,ib,ib2

!***********************************************************************
!     v=version
!     transmission_vS(ifrcase=6) v=uvw; only ifrcase is used
!     inside this subroutine

      if(cmch.eq.'thna') then
       theoryname='Mossbauer radioactive source        v=210'
       thfrcase(1)='Mossbauer transmission_vA(1)       v=210'
       thfrcase(2)='Mossbauer reemission_vA(2)         v=120'
       thfrcase(3)='Mossbauer transmission_vB(3)       v=210'
       thfrcase(4)='Mossbauer reemission_vB(4)         v=120'
       thfrcase(5)='Mossbauer transmission_vD(3)       v=200'
       thfrcase(6)='Mossbauer transmission_vS(6)       v=100'
       thfrcase(7)='Mossbauer transmission_vC(5)       v=200'
       return
      end if

!     version v=200:
!     the pointers have got a new sequence.
!     the second one is the end of the experimental data
!     ipt,"iposLG",ipTHS,ipTH,ipTHyy,ipSI,iaddr(3)
!     iposLG (pointer for array of the complex Lorentz-Gauss curve)
!     The routine tpg_lorgau.f90 stores parameters at iposLG-5,...,iposLG-1
!     in order to follow the alteration and necessary recalculation
!     of the Loentz-Gauss curve (case flag_SAMP=1)
!     If any constant is changed, all logical values are put to .false.,
!     such that effi recalculates all routines.
!***********************************************************************

      nchansp=int(channels)
      if(.not.b_data)then
         ichanks(0)=2; ichanks(1)=1; ichanks(2)=nchansp
      end if
      if(cmch.eq.'arr0') then
!     storage space
!      Data     info  nLG+1     th_A      th_exp    str_sub
!     |.........|.|..........|.........|.........|..........|
!     ipt nchanDa iposLG     ipTH     ipTHyy     ipSI

       nchanDA=3*nchansp  ! complex = 6*nchansp of real
                          ! for mfpt,mdfpt,mxpt,mvpt,mgpt,mdgpt
       iposLG=nchanDA+6   ! info=5 Lorentz-Gauss parameter, positon LG

       ipt=0              ! absolute address incremented in common/struct/
       ifrsto=0

       ibmisfit=1
       iflag_basis=int(flag_basis)
       km=nchansp/32
       km=8*km
       if(km.lt.128)km=128
       if(km.gt.512)km=512
       ib=int(convol)

       if (ib.gt.2*km) ib=2*km
       ib2=ib*2
       if(abs(vm_SC).gt.0.0) then
        kzl=int(km*vm_SC/abs(v_max))    ! km =: v_max
       else
        kzl=0
       end if
!       write(6,'(a,i5,2f10.2)') 'frame01 kzl=',kzl,vm_SC,v_max
       if(kzl.gt.0) then
        kzls=2*km
        kzla=kzl*2
        if(kzla.gt.2*km) kzla=2*km
       else
        kzla=2*km
        kzls=abs(kzl)*2
        if(kzls.gt.2*km) kzls=2*km
       end if

       sk2=kzls
       ak2=kzla
       sk8=sk2*4
       ak8=ak2*4

       if(kzla.gt.kzls) then
        nSC=sk2+ib2
        nAS=ak2+nSC
       else
        nAS=ak2+ib2
        nSC=sk2+nAS
       end if

       nLG=4*(nAS+nSC)
!       write(6,'(a,4i5)')'fr_mossbauer kzla,kzls,nSC,nAS=',kzla,kzls,nSC,nAS
        nchanLG=5+nLG+1         ! 5 for info; Lorentz-Gauss ; +1 dummy
        nchanTH=2*(km+1)            ! channels of the normalized spectrum

       if(ifrcase.eq.6) then
        nchanTHS=4*(2*nSC+1)+6*8! source refractionindex for selfabsorption
                                ! subroutine th_22indexsource
                                ! array_fr(nn+1)=cmplx(icase,anz)
                                ! array_fr(nn+2)=cmplx(mu_electronic,dsfoil)
                                ! array_fr(nn+3)=cmplx(d1_ov_e,b_d1ove)
                                ! array_fr(nn+4)=cmplx(mu_57Fe,b_mu57Fe)
                                ! array_fr(nn+5)=cmplx(mu_iron,b_muiron)
        ! array_fr(nn+6)=cmplx(b_source,123.123)
        ! 2 for icase, nz, thickness
        ! maximum in total 8 sites
        ifrsto=nchanDA+nchanLG+nchanTHS+nchanTH+nchansp/2+maxsub*(1+maxss)
        ipTHS=nchanDA+nchanLG
        ipTH=ipTHS+nchanTHS
        jpolsource=1
       else
        nchanTHS=0
        ipTHS=0
        ifrsto=nchanDA+nchanLG+nchanTH+nchansp/2+maxsub*(1+maxss)
        ipTH=nchanDA+nchanLG
        jpolsource=0
!       lstuct1case is assigned in the main routine mossbauer during execution
       end if
        ipTHyy=ipTH+nchanTH
        ipSI=ipTHyy+nchansp/2
        jpolabsorber=0
       return
      else if(cmch.eq.'arri') then
       ifrsto=0
       call l_structure(iss,lstruct1,lstruct2,lstruct3)
       if(nu_device(lstruct1).eq.1.and.lstruct3.gt.0) then
!       write(6,'(a,i1,a2,a)')'c3_prp(',lstruct3,')=',c3_prp(lstruct3)
        if(c3_prp(lstruct3).eq.'plrs') then
         jpolsource=1
        end if
       end if
       if(nu_device(lstruct1).eq.2) then
        if(lstruct2.gt.0) then
         if(lstruct3.eq.0) then

          if(istruct2(lstruct2)-istruct2(lstruct2-1).gt.1) then
!          storage for a layer with more than 1 site (storage for
!          a polarized layer )
           ifrsto=ifrsto+4*(nAS+1)
          end if
         else
          if(c3_prp(lstruct3).eq.'plrs') jpolabsorber=1
         end if
        else
!        storage for each device
         ifrsto=4*(nAS+1)+(km+1)
!        if there are more than one absorber (incoherent) the unweighted
!        spectrum of each absorber is stored
!         if(ifrcase.eq.5)ifrsto=ifrsto+(km+1)
        end if
       end if
       if(nu_device(lstruct1).eq.3.and.lstruct3.gt.0) then
!       detector is always assumed to be unpolarized
        ifrsto=nSC+1
       end if
       return
      else if(cmch.eq.'arre') then
       return
      end if         ! arr0,i,e
!    ****************************************************************
      if(.not.b_theory) then
!       absolute address of nstorSI is known here
        do k1=1,maxsub*(1+maxss)
         array_fr(ipSI+k1)=(0.,0.)
        end do
      end if
      if(Dabsorber.gt.888..or.Dabsorber.lt.0. )om_samp=0.
      ! there are theories which do not use the routines for geometry
      ! effects, so that om_samp is undefined/8888.8 respectively.
      ! The subroutine transmission_the2exp() calculates this effect if om_samp >=1

      if(km.gt.0)srw8=abs(v_max)/float(km)/8
      ! not very nice -> km,ipt are not yet defined when calling dstr
      mi=istruct2(istruct1(mstruct1))

!     storag on real_fr(2*(iarraysp+nstorFR)+m) (array_f)
!     corresponds to the i= drive_mode (label 100)
      mfpt=ipt*2                 ! y data from real_fr (factor 2)
      mdfpt=mfpt +nchansp        ! errors of y data (calculated by sqrt
      mvpt =mdfpt+nchansp        ! x data (velocity) drive_mode=3
      mgpt =mvpt +nchansp        ! velocity(i) calculated by drive
      mdgpt =mgpt +nchansp        ! geof(i) calculated by drive
      mxpt=mdgpt +nchansp        ! dfdgeo(i) calculated by drive
      thsum=-1.0
!     write(6,'(a,10i10)')'mfpt..',ipt,mfpt,mdfpt,mxpt,mvpt,mgpt,mdgpt
!     write(6,*) ipTH,nchanTH,ipTHyy,ipSI,iposLG,iaddr
!     write(6,*)  'channel_v0=',channel_v0,cmch
!   ******************************************************************
    select case(cmch)
    case('dstr')
       call moss_plot('dstr')
       call data2dread(0,info)
    case('data')
       k1=iss
       call data2dread(k1,info)
    case('ecic')
       call excinc2d(info)
    case('test')
       if(ifrcase.eq.2.or.ifrcase.eq.4)then
        call emission_the2exp(1,info)
       else
         call transmission_the2exp(1,info)
       end if
!      write(6,'(a)') ' after transmission_the2exp  test'
    case('dfdp')
!      derivatives df
       if(ifrcase.eq.2.or.ifrcase.eq.4)then
        call emission_the2exp(2,info)
       else
         call transmission_the2exp(2,info)
       end if
    case('subs')
       text(1)='           source'
       text(2)='         absorber'
       text(3)='  detector window'
       ch1=' '
       if(b_data) then
        ch132=name_data
       else
        ch132=name_theory
       end if
       call chcenter(ch132,ch1,ncl)
       cp132='plot of subspectra of'
       call chcenter(cp132,ch1,ncl)
       print '(a)',' '
       print '(a,/,a)', cp132(:ncl),ch132(:ncl)
       ch132='subspectra /added subspectra of'
       call chcenter(ch132,ch1,ncl)
       write(nwr,'(/,a)') ch132(:ncl)
       call moss_selsubspect(text)
    case('subt')
       text(1)='           source'
       text(2)='         absorber'
       text(3)='  detector window'
       print '(a)',' '
       print '(a)',' '
       print '(a)','                     subtraction of subspectra'
       call moss_selsubtract(text,isub)
       call moss_subtract('moss',mi,isub,info)
       k1=1
       if(ifrcase.eq.2.or.ifrcase.eq.4)then
        call emission_the2exp(k1,info)
       else
         call transmission_the2exp(k1,info)
       end if
       call moss_plot('subt')
       k1=0; k2=0
       if(b_theory) k1=1
       call xmosplo(k1,k2)
    case('plrf')
      k1=0
      if(b_theory) k1=1
      k2=0  ! 0: only plot and return, 1: interactive plot
      call xmosplo(k1,k2)
    case('plex')
      k1=0
      if(b_theory) k1=1
      k2=1  ! 0: only plot and return, 1: interactive plot
      call xmosplo(k1,k2)
    case('ple2')
!      plot execute
       k1=2   ! update test
       k2=1   ! 0: only plot and return, 1: interactive plot
       call xmosplo(k1,k2)
    case('ple3')
!      plot execute
       k1=3   ! update fit
       k2=1   ! 0: only plot and return, 1: interactive plot
       call xmosplo(k1,k2)
    case('dmod')
!      display mode , filling of plotarray
       call moss_plot('dmod')
    case('plot')
       if(b_theory) then
        k1=3
        if(ifrcase.eq.2.or.ifrcase.eq.4)then
         call emission_the2exp(k1,info)
        else
         call transmission_the2exp(k1,info)
        end if
        if(n_subset.gt.0) call moss_subspect('moss',mi,info)
       else
          print'(a,/,a)',' velocity scale not yet defined (command: te n)', &
          ' switch to channel scale by the command: dm sn d1'
       end if 
       call moss_plot('plot')
    case('punc')
!      punch data and theory
       k1=iss
       call punch2d(k1,info)
       cmch='dim2'
    end select
    return
    END

      subroutine sel_the2exp(cmch,n1,info)
        use comthe, only: ifrcase
      implicit none
      character(len=4), intent(in):: cmch
      integer,intent(in) :: n1
      integer,intent(out):: info
      integer       ien
      common/strLOC/ien(32)
      if(cmch.eq.'moss') then
       ien(32)=1
       if(ifrcase.eq.2.or.ifrcase.eq.4)then
        call emission_the2exp(n1,info)
       else
        call transmission_the2exp(n1,info)
       end if
      else if(cmch.eq.'stro') then
        call strob_the2exp(n1,info)
      end if
      return
      END



      subroutine transmission_the2exp(n1,info)
      use glbconst, only: maxfr1,maxfr2,maxn,maxthss,max8,icstrf,ivstrf,   &
                          infotext,enwr,nwr
      use comdat, only: mfpt,mxpt,mvpt,mgpt,mdgpt,b_theory,b_data,ichanks
      use comthe, only: real_fr,array_fr,thfk,dfdp,geof,ddpk,ib_df,kana,&
                        ibmisfit,thetaz,phiz
      implicit none
! *****************************************************************
!     n1 = 0 error return
!     n1 = 1 call of theory
!     n1 = 2 derivatives
!     n1 = 3 call of theory without mapping by the transformation
!            matrices by EFFI (important for subspectra)
! *****************************************************************
     integer, intent(in) :: n1
     integer, intent(out)::info
     integer, parameter:: illc=3*(maxfr1+1+maxfr2+maxthss)
     character(len=64) :: frthname
     integer, parameter:: nl=18
     integer       mstruct1,istruct1,istruct2,i_array,                          &
                   ipt,iposLG,ipTHS,ipTH,ipTHyy,iaddr
     common/struct/mstruct1,istruct1(0:maxfr1),istruct2(0:maxfr2),i_array(illc),&
                   frthname(2),                                                 &
                   ipt,iposLG,ipTHS,ipTH,ipTHyy,iaddr(3)

     integer       ibe,km,ien
     common/strLOC/ibe(2),km,ien(29)

     real          acsp,uthick_w,dum,channels,vm_SC,convol,th_dum,     &
                   drive_mode,channels_vmax,c_nonlinear,               &
                   om_samp,sc2aperture,abmue,Dsource,ring1to8,scmue,   &
                   Daperture,Dabsorber,sc2absorber,A_aperture,         &
                   B_aperture,phicone,alpha,dumcs
     common/acstrf/acsp,uthick_w,dum(3),channels,vm_SC,                &
                   convol,th_dum,drive_mode,channels_vmax,c_nonlinear, &
                   om_samp,sc2aperture,abmue,Dsource,ring1to8,scmue,   &
                   Daperture,Dabsorber,sc2absorber,A_aperture,         &
                   B_aperture,phicone,alpha,dumcs(icstrf-25)              


     real    avsp,f_factor,base,bg_fraction,v_max,                     &
             channel_v0,geo,vnl,                                       &
             dumv,ctauN,ctauP,dumvs
     common/avstrf/avsp,f_factor,base,bg_fraction,v_max,               &
             channel_v0,geo,vnl(nl),                                   &
             dumv,ctauN,ctauP,dumvs(ivstrf-10-nl)

     logical b_vsp,b_f_factor,b_base,b_bg_fraction,b_v_max,            &
             b_channel_v0,b_geo,b_vnl,                                 &
             b_dumv,b_ctauN,b_ctauP,b_dumvs
     common/bvstrf/b_vsp,b_f_factor,b_base,b_bg_fraction,b_v_max,      &
             b_channel_v0,b_geo,b_vnl(nl),         &
             b_dumv,b_ctauN,b_ctauP,b_dumvs(ivstrf-10-nl)


     real, dimension(-km-km/8:km+km/8):: yth,eth
     real, dimension(:), allocatable, save:: eps,cosa,a_r,phia
     real, dimension(:), allocatable, save:: velocity,dfdgeo
     real, dimension(:,:), allocatable:: ythz
     
     character(len=4):: enpr 
     integer nTHyy2

     logical b_total,b_fresonant,b_nonlinear,b_scale,b_drive,b_misfit,b_ctau
     real    dv,fa,ff,fresonant,h,vn,y1,y2,y3,y4,htau,tauN,tauP,   &
             hbase,sv,aa,bb,vm,Apt,Rpt
     integer i,j,k,ka,kad,kmd,l,nchansp,nomsamp,ndim,iring,igeo
     
     call mapsub(0,0,0)
      if(base.eq.0.) then
        info=1
        infotext(1)='<Mossbauer_spectrum>/parameter/global/base0'
        infotext(2)='**error** base = 0 '
        infotext(3)=' '
        return
      end if
      iring=int(ring1to8)
      if(iring.le.0)iring=1

      tauN=ctauN;  tauP=ctauP
      enpr=enwr(9) 

      nchansp=int(channels)
      i=size(velocity)
      if(allocated(velocity).and.i.ne.nchansp+2)deallocate(velocity,dfdgeo)
      if (.not.allocated(velocity)) &
           allocate(velocity(0:nchansp+1),dfdgeo(nchansp))
      nomsamp=int(om_samp)
      if(abs(nomsamp).gt.0)then
       ! calculation of ndim
       call geometryDIM(enpr,nomsamp,iring,Dabsorber,sc2absorber,Daperture, &
            sc2aperture,A_aperture,B_aperture,alpha,Dsource,geo,igeo,  &
            Apt,Rpt,ndim)

        if(enpr.eq.'<><>')then
        info=1
        infotext(1)='<Mossbauer_spectrum>/s-a-d_geometry'
        infotext(2)='**error** wrong geometry '
        infotext(3)=' '
        return
       end if

       k=size(cosa)
       if(.not.allocated(cosa))then
        allocate(eps(ndim),phia(ndim),cosa(ndim),a_r(ndim))
       else
        if(ndim.ne.k) then
         deallocate(eps,phia,cosa,a_r)
         allocate(eps(ndim),phia(ndim),cosa(ndim),a_r(ndim))
        end if 
       end if   
       if(nomsamp.lt.0)then
        k=km+km/8  
        if(.not.allocated(ythz))then
         allocate(ythz(-k:k,ndim))
        else
         if(ndim.ne.k) then
          deallocate(ythz)
          allocate(ythz(-k:k,ndim))
         end if 
        end if 
       end if
      end if   ! abs(nomsamp).gt.0

      if(.not.b_data.and.drive_mode.gt.2)drive_mode=1.

      h=abs(channel_v0/channels_vmax-1.0)*10
      if(h.gt.1.0.or.nchansp.gt.4*maxn.or.nchansp.gt.channels_vmax*4) then
       write(infotext(3),'(a,i5,2(a,f8.2))')                           &
        'nchansp=',nchansp,' channels_vmax=',channels_vmax,             &
        ' channel_v0=',channel_v0
       info=1+int(abs(10*h))
       infotext(1)='<Mossbauer_spectrum>/global_pa/ch_vmax/ch_v0'
       infotext(2)='**error** with one of the size of'//               &
                    ' channels,channels_vmax,channel_v0'
       infotext(3)=' '
       return
      end if

      ka=ipTH+1
      nTHyy2=ipTHyy*2
      kad=ka+km+1; kmd=km/8
      
      b_nonlinear=b_vnl(1)
      do j=2,nl
       b_nonlinear=b_nonlinear.or.b_vnl(j)
      end do
      b_scale=b_v_max.or.b_channel_v0.or.b_nonlinear
      b_drive=b_geo.or.b_scale
      b_ctau=b_ctauN.or.b_ctauP
      b_scale=b_scale.or.b_ctau
      b_fresonant=b_f_factor.or.b_bg_fraction
      b_drive=b_geo.or.b_scale


      fresonant=f_factor*(1.0-bg_fraction)
      
      if(fresonant.gt.1.)write(nwr,'(2a)')'**fresonant** larger than 1'

      if(b_drive)call drive(int(drive_mode),int(c_nonlinear),nchansp,mvpt,mgpt,&
                mdgpt,mxpt,channels_vmax,channel_v0,v_max,geo,vnl,info)
      if(info.gt.0)return
      do i=1,nchansp
       velocity(i)=real_fr(mvpt+i)
       geof(i)    =real_fr(mgpt+i)
       dfdgeo(i)  =real_fr(mdgpt+i)
      end do
      velocity(0)=velocity(1)-(velocity(2)-velocity(1))
      velocity(nchansp+1)=velocity(nchansp)+(velocity(nchansp)-velocity(nchansp-1))
!      write(6,'(/,a,4i8,8f15.2)')'km,mvpt,mgpt,mdgpt,base=',&
!                                  km,mvpt,mgpt,mdgpt,base,v_max
!      write(12,'(a,4g12.4)')' geo=',geo
!      write(6,'(8f10.5)')(geof(i),i=1,nchansp) 
!      write(6,'(a,4g12.4)')' velocity',abs(v_max)/float(km)
!      write(6,'(8f12.5)')(velocity(i),i=0,nchansp+1)
      thetaz=0
      call mossbauer_tmre(b_total,info)
      if(info.gt.0)return

    select case(n1)
    case(1)
!      -------------------calculation of thfk-----------------------
       if(b_base.or.b_fresonant.or.b_drive.or.b_total.or.b_ctau)then

        htau=exp(tauP)*(1.0+tauN)
        dv=abs(v_max)/float(km)          ! dv theory
        b_misfit=ibmisfit.eq.1
        if(nomsamp.gt.0) then
!        cosine smearing and aperture effects for scalar index of
!        refraction- Absorber perpendicular to the optical axis.
!        write(6,'(a)')' nomsamp.gt.1'
         do j =0,km
          yth(j)=1.-real(array_fr(ka+j)); yth(-j)=1.-aimag(array_fr(ka+j))
         end do
         do j =0,km
          eth(j)=-real(array_fr(kad+j)); eth(-j)=-aimag(array_fr(kad+j))        
         end do
!         write(6,'(6f12.5)')(yth(j),j=1,km,16)
!         write(6,'(6f12.5)')(eth(j),j=1,km,16)
         do k=1,kmd
          yth(-km-k)=yth(-km); yth(km+k) =yth(km)
         end do
         do k=1,kmd
          eth(-km-k)=eth(-km); eth(km+k) =eth(km)
         end do

         call geometry(igeo,nomsamp,iring,phicone,alpha,Apt,Rpt,A_aperture, &
                   ndim,-1.0,phia,cosa,a_r) 

         if(nomsamp.lt.0)then
            ythz(1,:)=yth            ! thetaz=0 optical axis
          do l=2,ndim
           phiz=phia(l); thetaz=acos(cosa(l))
           call mossbauer_tmre(b_total,info)
           do j=0,km
              ythz( l,j)=1.-real(array_fr(ka+j))
              ythz(l,-j)=1.-aimag(array_fr(ka+j))
           end do
          end do 
          do l=1,ndim           
           do k=1,kmd
            ythz(l,-km-k)=ythz(l,-km); ythz(l,km+k) =ythz(l,km)
           end do         
          end do  
         end if
       
         hbase=0.0 
         eps=1.0/cosa -1.0
         do l=1,ndim
          hbase=hbase+(1.0-eps(l)*abmue*(1.0-bg_fraction))*a_r(l)
         end do
         
         hbase=base/hbase*htau 
         do k=2,ichanks(0),2
          do j=ichanks(k-1),ichanks(k)
           call geometry(igeo,nomsamp,iring,phicone,alpha,Apt,Rpt,A_aperture, &
                   ndim,geof(j),phia,cosa,a_r)
           eps=1.0/cosa -1.0
           ff=0.0
           do l=1,ndim
            vn=velocity(j+1)*cosa(l)/dv; vm=velocity(j)*cosa(l)/dv
            i=int(vn);   if(i.lt.0)i=i-1
            vn=vn-float(i);  vm=vm-float(i); sv=vn+vm
            y1=yth(i-1)+eps(l)*eth(i-1)
            y2=yth(i)  +eps(l)*eth(i)
            y3=yth(i+1)+eps(l)*eth(i+1)
            y4=yth(i+2)+eps(l)*eth(i+2)
            aa=(y1+y4-y3-y2)/4.; bb=(y3-y2)-aa
            fa=y2 +(sv*(aa*sv+1.5*bb)-aa*vn*vm)/3.
            fa=1.-fa*fresonant
            ff=ff+(fa-eps(l)*abmue*(fa-bg_fraction))*a_r(l)
           end do
           thfk(j)=hbase*ff*exp(-ff*tauP)/(1.0+ff*tauN)
           if(b_misfit)dfdp(j)=base*geof(j)
          end do
         end do
         enwr(9)=enpr
        else
!        no cosine smearing
         hbase=base*htau
         do j =0,km
          yth(j)=1.-(1.-real(array_fr(ka+j)))*fresonant
          yth(-j)=1.-(1.-aimag(array_fr(ka+j)))*fresonant
         end do
         do k=1,km/8
          yth(-km-k)=yth(-km); yth(km+k) =yth(km)
         end do
         
         do k=2,ichanks(0),2
          do j=ichanks(k-1),ichanks(k)
           vn=velocity(j+1)/dv; vm=velocity(j)/dv
           if(vn.gt.vm) then
              i=int(vn);
           else
              i=int(vm)
          end if   
           if(i.lt.0)i=i-1
           vn=vn-float(i);  vm=vm-float(i); sv=vn+vm
           y1=yth(i-1)
           y2=yth(i)
           y3=yth(i+1)
           y4=yth(i+2)
           aa=(y1+y4-y3-y2)/4.; bb=(y3-y2)-aa
           ff=(y2 +(sv*(aa*sv+1.5*bb)-aa*vn*vm)/3.)*geof(j)
           thfk(j)=hbase*ff*exp(-ff*tauP)/(1.0+ff*tauN)
           dfdp(j)=base*geof(j)
          end do
         end do
        end if

        do j=1,nchansp
         real_fr(nTHyy2+j)=thfk(j)
        end do
       else
        do j=1,nchansp
         thfk(j)=real_fr(nTHyy2+j)
        end do
       end if
       b_theory=.true.
       if(enwr(16).eq.'sint') then
         y1=0.
         y2=0.0
         do k=2,ichanks(0),2
          do j=ichanks(k-1),ichanks(k)
           y1=y1+real_fr(mfpt+j)-thfk(j)
           y2=y2+(base*geof(j)-thfk(j))
          end do
         end do
         print'(a,f12.5)',' (theory-exp)/total_intensity:',y1/y2*100
        end if

      case(2)
!     ----------------------------derivatives----------------------
!     write(6,'(a,f15.5)') ' derivative step=',ddpk
      dv=abs(v_max)/float(km)
!     write(6,'(a,10L4)')'b_base,b_geo,b_scale,b_fresonant,b_f_factor=',  &
!                    b_base,b_geo,b_scale,b_fresonant,b_f_factor
      ib_df=0
      kana=0
      do j=1,nchansp
       thfk(j)=real_fr(nTHyy2+j)
      end do
!     parameter pl(j)
!     print'(a,5L3)','b_base,b_geo,b_scale,b_fresonant=',   &
!                     b_base,b_geo,b_scale,b_fresonant
!     variable pl(1)
      if(b_base) then
!       write(6,'(a,f10.1)')  'derivative of base -----base=',base
        do k=2,ichanks(0),2
         do j=ichanks(k-1),ichanks(k)
          dfdp(j)=thfk(j)/base
         end do
        end do
        kana=1
        ib_df=1
        goto 250
      end if

!     variable pl(3)
      if(b_geo) then
!       write(6,'(a)')  '   derivative of geo ------------'
!       in case of cosine smearing this derivation is approximate
        do k=2,ichanks(0),2
         do j=ichanks(k-1),ichanks(k)
          dfdp(j)=dfdgeo(j)*thfk(j)
         end do
        end do
        kana=1
        ib_df=1
        goto 250
      end if

      if(b_fresonant) then
       if(b_bg_fraction.and.b_f_factor) then
        write(nwr,'(a)')                                                &
        '**correlation_1** bg_fraction,f_factor are correlated by 1'
       end if
       if(b_bg_fraction) then
!       variable pl(2)
        do k=2,ichanks(0),2
         do j=ichanks(k-1),ichanks(k)
          dfdp(j)=-(thfk(j)-base*geof(j))/(1.0-bg_fraction)
         end do
        end do
        kana=1
        ib_df=1
        goto 250
       end if

       if(b_f_factor) then
!       variable pl(2)
!        write(6,'(a)')  '   derivative of f_factor ------------'
!        in case of cosine smearing Sum a_r(l) is replaced by geof (see above)
         do k=2,ichanks(0),2
          do j=ichanks(k-1),ichanks(k)
           dfdp(j)=(thfk(j)-base*geof(j))/f_factor
          end do
         end do
         kana=1
         ib_df=1
         goto 250
        end if
       end if

       kana=0
       htau=exp(tauP)*(1.0+tauN)
       if(nomsamp.gt.0) then
!       cosine smearing and aperture effects
         do j =0,km
          yth(j)=1.-real(array_fr(ka+j)); yth(-j)=1.-aimag(array_fr(ka+j))
         end do
         ka=ka+km+1
         do j =0,km
          eth(j)=-real(array_fr(ka+j)); eth(-j)=-aimag(array_fr(ka+j))        
         end do
         do k=1,km/8
          yth(-km-k)=yth(-km); yth(km+k) =yth(km)
         end do
         do k=1,km/8
          eth(-km-k)=eth(-km); eth(km+k) =eth(km)
         end do

         call geometry(igeo,nomsamp,iring,phicone,alpha,Apt,Rpt,A_aperture, &
                   ndim,-1.0,phia,cosa,a_r) 

         ndim=ndim
         hbase=0.0 
         eps=1.0/cosa -1.0
         do l=1,ndim
          hbase=hbase+(1.0-eps(l)*abmue*(1.0-bg_fraction))*a_r(l)
         end do
         
         hbase=base/hbase*htau 
         do k=2,ichanks(0),2
          do j=ichanks(k-1),ichanks(k)
           call geometry(igeo,nomsamp,iring,phicone,alpha,Apt,Rpt,A_aperture, &
                   ndim,geof(j),phia,cosa,a_r)
           eps=1.0/cosa -1.0
           ff=0.0
           do l=1,ndim
            vn=velocity(j+1)*cosa(l)/dv; vm=velocity(j)*cosa(l)/dv
            i=int(vn);   if(i.lt.0)i=i-1
            vn=vn-float(i);  vm=vm-float(i); sv=vn+vm
            y1=yth(i-1)+eps(l)*eth(i-1)
            y2=yth(i)  +eps(l)*eth(i)  
            y3=yth(i+1)+eps(l)*eth(i+1)
            y4=yth(i+2)+eps(l)*eth(i+2)
            aa=(y1+y4-y3-y2)/4.; bb=(y3-y2)-aa
            fa=y2 +(sv*(aa*sv+1.5*bb)-aa*vn*vm)/3.
            fa=1.-fa*fresonant
            ff=ff+(fa-eps(l)*abmue*(fa-bg_fraction))*a_r(l)
           end do
           dfdp(j)=hbase*ff*exp(-ff*tauP)/(1.0+ff*tauN)
          end do
         end do

         do k=2,ichanks(0),2
          do j=ichanks(k-1),ichanks(k)
             dfdp(j)=(dfdp(j)-thfk(j))/ddpk
          end do
         end do
       else
!       no cosine smearing
        hbase=base*htau
        do j =0,km
         yth(j)=1.-(1.-real(array_fr(ka+j)))*fresonant
         yth(-j)=1.-(1.-aimag(array_fr(ka+j)))*fresonant
        end do
        do k=1,km/8
         yth(-km-k)=yth(-km); yth(km+k) =yth(km)
        end do

        do k=2,ichanks(0),2
         do j=ichanks(k-1),ichanks(k)
          vn=velocity(j+1)/dv; vm=velocity(j)/dv
          if(vn.gt.vm) then
           i=int(vn);
          else
           i=int(vm)
          end if   
          if(i.lt.0)i=i-1
          vn=vn-float(i);  vm=vm-float(i); sv=vn+vm
          y1=yth(i-1)
          y2=yth(i)
          y3=yth(i+1)
          y4=yth(i+2)
          aa=(y1+y4-y3-y2)/4.; bb=(y3-y2)-aa
          ff=(y2 +(sv*(aa*sv+1.5*bb)-aa*vn*vm)/3.)*geof(j)
          dfdp(j)=hbase*ff*exp(-ff*tauP)/(1.0+ff*tauN)
         end do
        end do
        do k=2,ichanks(0),2
         do j=ichanks(k-1),ichanks(k)
           dfdp(j)=(dfdp(j)-thfk(j))/ddpk
         end do
        end do
       end if   ! nomsamp
       ib_df=1

  250 continue
!      write(6,'(4e15.6)') (dfdp(i),i=1,nchansp,32)
  case(3)
     if(b_theory)then
      if(b_base.or.b_fresonant.or.b_drive.or.b_total.or.b_ctau)then
       dv=abs(v_max)/float(km)
!      this label address is used for the calculation of subspectra

       htau=exp(tauP)*(1.0+tauN)
       if(nomsamp.gt.0) then
!       cosine smearing and aperture effects
         do j =0,km
          yth(j)=1.-real(array_fr(ka+j)); yth(-j)=1.-aimag(array_fr(ka+j))
         end do
         ka=ka+km+1
         do j =0,km
          eth(j)=-real(array_fr(ka+j)); eth(-j)=-aimag(array_fr(ka+j))        
         end do
         do k=1,km/8
          yth(-km-k)=yth(-km); yth(km+k) =yth(km)
         end do
         do k=1,km/8
          eth(-km-k)=eth(-km); eth(km+k) =eth(km)
         end do

         call geometry(igeo,nomsamp,iring,phicone,alpha,Apt,Rpt,A_aperture, &
                   ndim,-1.0,phia,cosa,a_r) 

         hbase=0.0 
         eps=1.0/cosa -1.0
         do l=1,ndim
          hbase=hbase+(1.0-eps(l)*abmue*(1.0-bg_fraction))*a_r(l)
         end do
         
         hbase=base/hbase*htau 
         do j=1,nchansp
           call geometry(igeo,nomsamp,iring,phicone,alpha,Apt,Rpt,A_aperture, &
                   ndim,geof(j),phia,cosa,a_r)
          eps=1.0/cosa -1.0
          ff=0.0
          do l=1,ndim
           vn=velocity(j+1)*cosa(l)/dv; vm=velocity(j)*cosa(l)/dv
           i=int(vn);   if(i.lt.0)i=i-1
           vn=vn-float(i);  vm=vm-float(i); sv=vn+vm
           y1=yth(i-1)+eps(l)*eth(i-1)
           y2=yth(i)  +eps(l)*eth(i)  
           y3=yth(i+1)+eps(l)*eth(i+1)
           y4=yth(i+2)+eps(l)*eth(i+2)
           aa=(y1+y4-y3-y2)/4.; bb=(y3-y2)-aa
           fa=y2 +(sv*(aa*sv+1.5*bb)-aa*vn*vm)/3.
           fa=1.-fa*fresonant
           ff=ff+(fa-eps(l)*abmue*(fa-bg_fraction))*a_r(l) 
          end do
          thfk(j)=hbase*ff*exp(-ff*tauP)/(1.0+ff*tauN)
         end do
       else
        !       no cosine smearing
        hbase=base*htau  
        do j =0,km
         yth(j)=1.-(1.-real(array_fr(ka+j)))*fresonant
         yth(-j)=1.-(1.-aimag(array_fr(ka+j)))*fresonant
        end do
        do k=1,km/8
         yth(-km-k)=yth(-km); yth(km+k) =yth(km)
        end do
        do k=2,ichanks(0),2
         do j=ichanks(k-1),ichanks(k)
          vn=velocity(j+1)/dv; vm=velocity(j)/dv
          i=int(vn); if(i.lt.0)i=i-1
          vn=vn-float(i);  vm=vm-float(i); sv=vn+vm
          y1=yth(i-1)
          y2=yth(i)
          y3=yth(i+1)
          y4=yth(i+2)
          aa=(y1+y4-y3-y2)/4.; bb=(y3-y2)-aa
          ff=(y2 +(sv*(aa*sv+1.5*bb)-aa*vn*vm)/3.)*geof(j)
          thfk(j)=hbase*ff*exp(-ff*tauP)/(1.0+ff*tauN)
        end do
       end do
      end if  ! if(nomsamp.gt.0)
     else
      do j=1,nchansp
       thfk(j)=real_fr(nTHyy2+j)
      end do
     end if  ! if(b_base.or
     end if   ! if(b_theory)then
   end select
   return
  END subroutine transmission_the2exp

     subroutine emission_the2exp(n1,info)
      use glbconst, only: maxfr1,maxfr2,maxn,maxthss,max8,icstrf,ivstrf,   &
                          infotext,enwr
      use comdat, only: mfpt,mxpt,mvpt,mgpt,mdgpt,b_theory,b_data,ichanks
      use comthe, only: real_fr,array_fr,thfk,dfdp,geof,ddpk,ib_df, &
                        kana,ibmisfit,thsum
      implicit none
! *****************************************************************
!     n1 = 0 error return
!     n1 = 1 call of theory
!     n1 = 2 derivatives
!     n1 = 3 call of theory without mapping by the transformation
!            matrices by EFFI (important for subspectra)
! *****************************************************************
     integer, intent(in) :: n1
     integer, intent(out)::info
     integer, parameter:: illc=3*(maxfr1+1+maxfr2+maxthss)
     character(len=64) :: frthname
     integer, parameter:: nl=18
     integer       mstruct1,istruct1,istruct2,i_array,                  &
                   ipt,iposLG,ipTHS,ipTH,ipTHyy,iaddr
     common/struct/mstruct1,istruct1(0:maxfr1),istruct2(0:maxfr2),      &
                   i_array(illc),frthname(2),                           &
                   ipt,iposLG,ipTHS,ipTH,ipTHyy,iaddr(3)

     integer       ibe,km,ien
     common/strLOC/ibe(2),km,ien(29)

     real     acsp,uthick_w,dum,channels,vm_SC,convol,th_dum,     &
              drive_mode,channels_vmax,c_nonlinear,               &
              om_samp,sc2aperture,abmue,Dsource,ring1to8,scmue,   &
              Daperture,Dabsorber,sc2absorber,A_aperture,         &
              B_aperture,phicone,alpha,dumcs
     common/acstrf/acsp,uthick_w,dum(3),channels,vm_SC,                &
                   convol,th_dum,drive_mode,channels_vmax,c_nonlinear, &
                   om_samp,sc2aperture,abmue,Dsource,ring1to8,scmue,   &
                   Daperture,Dabsorber,sc2absorber,A_aperture,         &
                   B_aperture,phicone,alpha,dumcs(icstrf-25)


     real    avsp,f_factor,base,counts,v_max,                     &
             channel_v0,geo,vnl,                                       &
             dumv,ctauN,ctauP,dumvs
     common/avstrf/avsp,f_factor,base,counts,v_max,               &
             channel_v0,geo,vnl(nl),                                   &
             dumv,ctauN,ctauP,dumvs(ivstrf-10-nl)

     logical b_vsp,b_f_factor,b_base,b_counts,b_v_max,            &
             b_channel_v0,b_geo,b_vnl,                                 &
             b_dumv,b_ctauN,b_ctauP,b_dumvs
     common/bvstrf/b_vsp,b_f_factor,b_base,b_counts,b_v_max,      &
             b_channel_v0,b_geo,b_vnl(nl),         &
             b_dumv,b_ctauN,b_ctauP,b_dumvs(ivstrf-10-nl)

     real, dimension(-640:640):: yth
     real, dimension(:), allocatable, save:: phia,cosa,a_r
     real, dimension(:), allocatable, save:: velocity,dfdgeo

     character(len=4):: enpr
     integer nTHyy2

     logical b_total,b_fresonant,b_nonlinear,b_scale,b_drive,b_misfit,b_ctau
     real    dv,fa,ff,fresonant,h,vn,y1,y2,y3,y4,htau,tauN,tauP,Apt,Rpt,  &
             sv,aa,bb,vm
     integer i,j,k,ka,kmd,l,nchansp,nomsamp,ndim,iring,igeo

     call mapsub(0,0,0)
      if(base.eq.0.) then
        info=1
        infotext(1)='<Mossbauer_spectrum>/parameter/global/base0'
        infotext(2)='**error** base = 0 '
        infotext(3)=' '
        return
      end if
      iring=int(ring1to8)
      tauN=ctauN;  tauP=ctauP

      nchansp=int(channels)
      i=size(velocity)
      if(allocated(velocity).and.i.ne.nchansp+2) &
                           deallocate(velocity,dfdgeo)
      if (.not.allocated(velocity)) &
               allocate(velocity(0:nchansp+1),dfdgeo(nchansp))
       nomsamp=int(om_samp)
       if(abs(nomsamp).gt.0)then
        ! calculation of ndim
        call geometryDIM(enpr,nomsamp,iring,Dabsorber,sc2absorber,Daperture, &
            sc2aperture,A_aperture,B_aperture,alpha,Dsource,geo,igeo,  &
            Apt,Rpt,ndim)

        if(enpr.eq.'<><>')then
         info=1
         infotext(1)='<Mossbauer_spectrum>/s-a-d_geometry'
         infotext(2)='**error** wrong geometry '
         infotext(3)=' '
         return
        end if
     
        k=size(cosa)
        if(.not.allocated(cosa))then
         allocate(phia(ndim),cosa(ndim),a_r(ndim))
        else
         if(ndim.ne.k) then
          deallocate(phia,cosa,a_r)
          allocate(phia(ndim),cosa(ndim),a_r(ndim))
         end if 
        end if
       end if   ! abs(nomsamp).gt.0      

       if(.not.b_data.and.drive_mode.gt.2)drive_mode=1.

       h=abs(channel_v0/channels_vmax-1.0)*10
       if(h.gt.1.0.or.nchansp.gt.4*maxn.or.nchansp.gt.channels_vmax*4) then
        write(infotext(3),'(a,i5,2(a,f8.2))')                           &
        'nchansp=',nchansp,' channels_vmax=',channels_vmax,             &
        ' channel_v0=',channel_v0
        info=1+int(abs(10*h))
        infotext(1)='<Mossbauer_spectrum>/global_pa/ch_vmax/ch_v0'
        infotext(2)='**error** with one of the size of'//               &
                    ' channels,channels_vmax,channel_v0'
        infotext(3)=' '
        return
       end if

      ka=ipTH+1
      nTHyy2=ipTHyy*2 
      kmd=km/8
      
      b_nonlinear=b_vnl(1)
      do j=2,nl
       b_nonlinear=b_nonlinear.or.b_vnl(j)
      end do
      b_scale=b_v_max.or.b_channel_v0.or.b_nonlinear
      b_drive=b_geo.or.b_scale
      b_ctau=b_ctaun.or.b_ctauP
      b_scale=b_scale.or.b_ctau
      b_fresonant=b_f_factor.or.b_counts
      b_drive=b_geo.or.b_scale

      fresonant=f_factor*counts

      if(b_drive)call drive(int(drive_mode),int(c_nonlinear),nchansp,mvpt,mgpt,&
                mdgpt,mxpt,channels_vmax,channel_v0,v_max,geo,vnl,info)
      if(info.gt.0)return
      do i=1,nchansp
       velocity(i)=real_fr(mvpt+i)
       geof(i)    =real_fr(mgpt+i)
       dfdgeo(i)  =real_fr(mdgpt+i)
      end do
      velocity(0)=velocity(1)-(velocity(2)-velocity(1))
      velocity(nchansp+1)=velocity(nchansp)+(velocity(nchansp)-velocity(nchansp-1))

      call mossbauer_tmre(b_total,info)
      if(info.gt.0)return

      ff=0.0
      do j=0,km
         yth(j)=(real(array_fr(ka+j))-1.0)
         yth(-j)=(aimag(array_fr(ka+j))-1.0)
         ff=ff+yth(j)+yth(-j)
      end do
     if(thsum.lt.0.) then
      thsum=ff
      ff=counts/ff
     else
      ff=-counts/thsum
     end if    
      do j=-km,km
       yth(j)=yth(j)*ff   
      end do
      do k=1,kmd
       yth(-km-k)=yth(-km); yth(km+k) =yth(km)
      end do

    
      htau=exp(tauP)*(1.0+tauN)
!     write(6,'(8f10.3)')(yth(j),j=-km,km,16)

 select case(n1)
    case(1)
!      -------------------calculation of thfk-----------------------
      if(b_base.or.b_drive.or.b_total.or.b_ctau)then

        dv=abs(v_max)/float(km)
        b_misfit=ibmisfit.eq.1
       if(nomsamp.gt.0) then
!        cosine smearing and aperture effects for scalar index of
!        refraction- Absorber perpendicular to the optical axis.
!        write(6,'(a)')' nomsamp.gt.1'
         enpr=enwr(9)
         base=base*htau
       
         call geometry(igeo,nomsamp,iring,phicone,alpha,Apt,Rpt,A_aperture, &
              ndim,-1.0,phia,cosa,a_r)

         do k=2,ichanks(0),2
          do j=ichanks(k-1),ichanks(k)
           call geometry(igeo,nomsamp,iring,phicone,alpha,Apt,Rpt,A_aperture, &
                   ndim,geof(j),phia,cosa,a_r)
           ff=0.0
           do l=1,ndim
            vn=velocity(j+1)*cosa(l)/dv; vm=velocity(j)*cosa(l)/dv
            i=int(vn);  if(i.lt.0)i=i-1
            vn=vn-float(i);  vm=vm-float(i); sv=vn+vm
            y1=yth(i-1)
            y2=yth(i)  
            y3=yth(i+1)
            y4=yth(i+2)
            aa=(y1+y4-y3-y2)/4.; bb=(y3-y2)-aa
            fa=y2 +(sv*(aa*sv+1.5*bb)-aa*vn*vm)/3.
            ff=ff+fa*a_r(l)
           end do
           ff=ff+base*geof(j)
           thfk(j)=ff*htau*exp(-ff*tauP)/(1.0+ff*tauN)
           if(b_misfit)dfdp(j)=geof(j)
          end do
         end do
         enwr(9)=enpr
       else
!       no cosine smearing
!       array_fr(ka+j) are already multiplied by wi(isite) [counts]
        
        do k=2,ichanks(0),2
         do j=ichanks(k-1),ichanks(k)
          vn=velocity(j+1)/dv; vm=velocity(j)/dv
          i=int(vn);   if(i.lt.0)i=i-1
          vn=vn-float(i);  vm=vm-float(i); sv=vn+vm
          y1=yth(i-1)
          y2=yth(i)
          y3=yth(i+1)
          y4=yth(i+2)
          aa=(y1+y4-y3-y2)/4.; bb=(y3-y2)-aa
          ff=(y2 +(sv*(aa*sv+1.5*bb)-aa*vn*vm)/3.+base)*geof(j)
          thfk(j)=ff*htau*exp(-ff*tauP)/(1.0+ff*tauN)
          if(b_misfit)dfdp(j)=base*geof(j)
         end do
        end do
       end if ! cosine smearing
       do j=1,nchansp
        real_fr(nTHyy2+j)=thfk(j)
       end do
      else
       do j=1,nchansp
        thfk(j)=real_fr(nTHyy2+j)
       end do
      end if
!      write(6,'(4e15.6)') (thfk(j),j=1,nchansp,64)
      b_theory=.true.
    

   case(2)
!     ----------------------------derivatives----------------------
      dv=abs(v_max)/float(km)
      ib_df=0
      kana=0
      do j=1,nchansp
       thfk(j)=real_fr(nTHyy2+j)
      end do
!     parameter pl(j)

!     variable pl(1)
      if(b_base) then
!        write(6,'(a,f10.1)')  'derivative of base -----base=',base
        do k=2,ichanks(0),2
         do j=ichanks(k-1),ichanks(k)
          dfdp(j)=geof(j)
         end do
        end do
        kana=1
        ib_df=1
        goto 250
      end if

!     variable pl(3)
      if(b_geo) then
!       write(6,'(a)')  '   derivative of geo ------------'
!       in case of cosine smearing this derivation is approximate
        do k=2,ichanks(0),2
         do j=ichanks(k-1),ichanks(k)
          dfdp(j)=dfdgeo(j)*thfk(j)/geof(j)
         end do
        end do
        kana=1
        ib_df=1
        goto 250
      end if


      kana=0
      if(nomsamp.gt.0) then
!      cosine smearing and aperture effects
       call geometry(igeo,nomsamp,iring,phicone,alpha,Apt,Rpt,A_aperture, &
                   ndim,-1.0,phia,cosa,a_r) 
       
       do k=2,ichanks(0),2
         do j=ichanks(k-1),ichanks(k)
          call geometry(igeo,nomsamp,iring,phicone,alpha,Apt,Rpt,A_aperture, &
                   ndim,geof(j),phia,cosa,a_r)
          ff=0.0
          do l=1,ndim
           vn=velocity(j+1)*cosa(l)/dv; vm=velocity(j)*cosa(l)/dv
           i=int(vn);  if(i.lt.0)i=i-1
           vn=vn-float(i);  vm=vm-float(i); sv=vn+vm
           y1=yth(i-1)
           y2=yth(i)  
           y3=yth(i+1)
           y4=yth(i+2)
           aa=(y1+y4-y3-y2)/4.; bb=(y3-y2)-aa
           fa=y2 +(sv*(aa*sv+1.5*bb)-aa*vn*vm)/3.
           ff=ff+fa*a_r(l)
          end do
          ff=base*geof(j)+ff
          dfdp(j)=(ff*htau*exp(-ff*tauP)/(1.0+ff*tauN)-thfk(j))/ddpk
          end do
        end do
      else
!       no cosine smearing
        do k=2,ichanks(0),2
         do j=ichanks(k-1),ichanks(k)
          vn=velocity(j+1)/dv; vm=velocity(j)/dv
          i=int(vn);   if(i.lt.0)i=i-1
          vn=vn-float(i);  vm=vm-float(i); sv=vn+vm
          y1=yth(i-1)
          y2=yth(i)
          y3=yth(i+1)
          y4=yth(i+2)
          aa=(y1+y4-y3-y2)/4.; bb=(y3-y2)-aa
          ff=(y2 +(sv*(aa*sv+1.5*bb)-aa*vn*vm)/3.+base)*geof(j)
          dfdp(j)=(ff*htau*exp(-ff*tauP)/(1.0+ff*tauN)-thfk(j))/ddpk
         end do
        end do
      end if ! cosine smearing
      ib_df=1

  250  continue
!      write(6,'(4e15.6)') (dfdp(i),i=1,nchansp,32)
   case(3)
    if(b_theory)then
     if(b_base.or.b_fresonant.or.b_drive.or.b_total.or.b_ctau)then
      dv=abs(v_max)/float(km)
!     this label address is used for the calculation of subspectra

      if(nomsamp.gt.0) then
!      cosine smearing and aperture effects
        do j=1,nchansp
         call geometry(igeo,nomsamp,iring,phicone,alpha,Apt,Rpt,A_aperture, &
               ndim,geof(j),phia,cosa,a_r)
         ff=0.0
         do l=1,ndim
          vn=velocity(j+1)*cosa(l)/dv; vm=velocity(j)*cosa(l)/dv
          i=int(vn);  if(i.lt.0)i=i-1
          vn=vn-float(i);  vm=vm-float(i); sv=vn+vm
          y1=yth(i-1)
          y2=yth(i)  
          y3=yth(i+1)
          y4=yth(i+2)
          aa=(y1+y4-y3-y2)/4.; bb=(y3-y2)-aa
          fa=y2 +(sv*(aa*sv+1.5*bb)-aa*vn*vm)/3.
          ff=ff+fa*a_r(l)
         end do
         ff=base*geof(j)+ff
         thfk(j)=ff*htau*exp(-ff*tauP)/(1.0+ff*tauN)        
        end do
      else
!       no cosine smearing
        do j=1,nchansp
         vn=velocity(j+1)/dv; vm=velocity(j)/dv
         i=int(vn);   if(i.lt.0)i=i-1
         vn=vn-float(i);  vm=vm-float(i); sv=vn+vm
         y1=yth(i-1)
         y2=yth(i)
         y3=yth(i+1)
         y4=yth(i+2)
         aa=(y1+y4-y3-y2)/4.; bb=(y3-y2)-aa
         ff=(y2 +(sv*(aa*sv+1.5*bb)-aa*vn*vm)/3.+base)*geof(j)
         thfk(j)=ff*htau*exp(-ff*tauP)/(1.0+ff*tauN) 
        end do
      end if    ! nomsamp.gt.0
     else
       do j=1,nchansp
        thfk(j)=real_fr(nTHyy2+j)
       end do
     end if   ! if(base
  end if    ! if(theory

   end select
!  END if
End subroutine emission_the2exp



!     H. Spiering January,1995 Mainz
!                 oktober 2004 Budapest, Dezember Christcurch
!                 June Mainz 2005
!     moving absorber or moving source
!
     subroutine mossbauer_tmre(b_total,info)
     use glbconst, only: maxfr1,maxfr2,maxn,maxscrd,maxthss, &
                         ivstrf,ivfr1trf,ivfr2trf,ivsstrf, &
                         icstrf,icfr1trf,icfr2trf,icsstrf, &
                         ipostxtu,enwr,nwr,nrd
     use comstr, only: nu_device
     use comthe, only: array_fr,ifrcase,lstr1case
     implicit none
     logical, intent(out):: b_total
     integer, intent(out):: info

     integer       mstruct1,istruct1,istruct2,l1_get,l2_get,l3_get,    &
                   l1_put,l2_put,l3_put,ipt,iposLG,ipTHS,ipTH,iaddr
     character(len=4) :: c1_prp,c2_prp,c3_prp
     character(len=64):: frthname
     common/struct/mstruct1,istruct1(0:maxfr1),istruct2(0:maxfr2),     &
                   l1_get(0:maxfr1),l2_get(maxfr2),l3_get(maxthss),    &
                   l1_put(0:maxfr1),l2_put(maxfr2),l3_put(maxthss),    &
                   c1_prp(0:maxfr1),c2_prp(maxfr2),c3_prp(maxthss),    &
                   frthname(2),                                        &
                   ipt,iposLG,ipTHS,ipTH,iaddr(4)

     integer       ifrth,km,idum,nAS,nSC,ien,lstruct3
     common/strLOC/ifrth(2),km,idum(12),nAS,nSC,ien(14),lstruct3

     real          acsp,uthick_w,dum,channels,vm_SC,convol,       &
                   th_dum,drive_mode,channels_vmax,c_nonlinear,om_samp,&
                   dumcs
     common/acstrf/acsp,uthick_w,dum(3),channels,vm_SC,convol,&
                   th_dum,drive_mode,channels_vmax,c_nonlinear,om_samp,&
                   dumcs(icstrf-13)
     real           avsp,f_factor,base,bg_fraction,dumvs
     common/avstrf/avsp,f_factor,base,bg_fraction,dumvs(ivstrf-4)

     logical       b_vsp,b_v_max,b_dumvsp
     common/bvstrf/b_vsp(4),b_v_max,b_dumvsp(ivstrf-5)

     real            avfr1,devfactor, devHext,devthetaHe,devphiHe,     &
                     dvfr1
     common/avfr1trf/avfr1,devfactor,devHext,devthetaHe,devphiHe,      &
                     dvfr1(ivfr1trf-5)
     logical         b_vfr1,b_devfactor,b_devHext,b_devthetaHe,        &
                     b_devphiHe,b_dvfr1
     common/bvfr1trf/ b_vfr1,b_devfactor,b_devHext,b_devthetaHe,       &
                      b_devphiHe,b_dvfr1(ivfr1trf-5)

     real            acfr2,dumcfr2
     common/acfr2trf/acfr2,dumcfr2(icfr2trf-1)

!                    phi,theta,psi are not directly assigned
!                    but by arry_fr via the routine orientation
     real            avfr2,th_factor,dumx,Hext,thetaHe,phiHe,          &
                     phi,theta,psi,dumvfr2
     common/avfr2trf/avfr2,th_factor,dumx(4),Hext,thetaHe,phiHe,       &
                     phi,theta,psi,dumvfr2(ivfr2trf-12)
     logical         b_vfr2,b_thfactor,b_dumx,b_phi,b_theta,b_psi,     &
                     b_dumvfr2
     common/bvfr2trf/b_vfr2,b_thfactor,b_dumx(7),b_phi,b_theta,b_psi,  &
                     b_dumvfr2(ivfr2trf-12)

     real            vsstrf,sftr
     common/avsstrf/ vsstrf(ivsstrf)
     logical         b_vsstrf,b_not_sftr
     common/bvsstrf/ b_vsstrf(ivsstrf)

     integer, parameter:: mAS=4*maxn,                                    &
                         idumscrd=maxscrd-10*(2*mAS+1)
     real        sTr,sP1,sP2,sP3,aTr,aP1,aP2,aP3,eTr,sa,dumscrd
     common/scrd/sTr(-mAS:mAS),sP1(-mAS:mAS),sP2(-mAS:mAS),sP3(-mAS:mAS),&
                 aTr(-mAS:mAS),aP1(-mAS:mAS),aP2(-mAS:mAS),aP3(-mAS:mAS),&
                 eTr(-mAS:mAS),sa(-mAS:mAS),dumscrd(idumscrd)
     complex     yout(0:512)

     logical   b_source,b_absorber,b_detector,b_lay,                   &
               b_polabsorber,b_polsource,b_absdetector,b_reemission
     logical,dimension(maxfr1)    :: b_device
     logical,dimension(maxfr2)    :: b_module
     logical,dimension(0:maxthss) :: b_polisite

     integer   icase,issa,idt,i,j,n,na,nn,n8,n2,mfr1,nspdv,            &
               k,lay,layb,layl,nabs0,nabs1,nabs2,nabs3,                &
               npolSCAS,iyout,isubthe,                                 &
               lstruct1,lstruct2,isse,iget,isite,isource

     real  satrexpr,factor_thickness,h,wixf,hnorm,attenuation,f_eff,ff
     real, save :: finish,start,wfinish,wstart
     real,   dimension(maxfr1) :: areafrac
     real,   dimension(maxthss):: wi
     complex,dimension(4)      :: Arho
     complex  app,apm,amp,satraceh
     complex,dimension(4,maxfr2):: aam
     complex,dimension(4)::  aa,bb,dd   !,cc
     real,   dimension(8)::  ri,anorm
     equivalence(dd(1),ri(1))
     complex(kind=8),dimension(4)::  daa,dbb,dcc
     real,    dimension(:), allocatable :: detector
     complex, dimension(:), allocatable :: scfunction

     complex, parameter:: C0=cmplx(0.0,0.0), hci=cmplx(0.0,1.0), &
                                             hc1=cmplx(1.0,0.0)
     character(len=132) :: querytype,chcom
     character(len=1):: ch1
     logical b_pp

      allocate(detector(-nSC:nSC))
      eTr=0.0
      do i=1,maxthss
       wi(i)=0.
      end do
!     ch_case='source','absorber','detector',
!     ifrcase=1,3,5,6 transmission, 2.4 reemission
!     ifrcase=5   incoherent absorber

      b_reemission=.false.
      if(ifrcase.eq.2.or.ifrcase.eq.4) b_reemission=.true.
      b_polsource  =.false.
      b_polabsorber=.false.

      b_source =.false.
      b_absorber=.false.
      b_detector=.false.
      b_absdetector=.false.
!-----------------------------------------------------------------
! why this lines  isubthe=1 .... b_total= b_total.or.(isubthe.gt.0)
!-------------------------------------------------------------------

      b_total=b_v_max
      isubthe=1
      call get_theory(isubthe,info)
      if(info.gt.0) goto 1000
      b_total= b_total.or.(isubthe.gt.0)
!      write(6,'(a,i3)')'ifrcase=',ifrcase
!      write(6,'(a,i3)')'mstruct1=',mstruct1
!      write(6,'(a,10i3)')'istruct1=',(istruct1(i),i=0,5)
!      write(6,'(a,10i3)')'istruct2=',(istruct2(i),i=0,5)

      do lstruct1=1,mstruct1
       b_device(lstruct1)=.false.
       call mapsub(lstruct1,0,0)
       isubthe=1     ! frame
       call get_theory(isubthe,info)
       if(info.gt.0) goto 1000
       b_total= b_total.or.(isubthe.gt.0)

       if(nu_device(lstruct1).eq.1) then
        lstr1case=1     ! entry for the source is transferred by the
!                         common block  'common/struct/' to the
!                         the routine called by get_theory
!                         the subroutine th_..index..() branches to
!                         source,absorber,detector by the command
!                         goto (100,200,300),lstr1case
!      -----------------------------------------------------------------
!      layer 1 = source
!      -----------------------------------------------------------------
!      Sums of source curves at the positions Eagd. For the convolution
!      integral the stepwidth is srw2. The stepwidth srw8 of the
!      source function guarantees that the source line is exact
!      within 1/8 channel(flag_SAMP=1).
!
!      ----0--------------------------nLG
!       nLG--------------------------0-------------        srw8
!        nSC------------0------------nSC                    srw2

        if(ifrcase.eq.6)then
         nn=4*(2*nSC+1)
         allocate(scfunction(nn))
         scfunction=C0
        end if
        
        do lstruct2=istruct1(lstruct1-1)+1,istruct1(lstruct1)
         ipostxtu=lstruct2
           
         call mapsub(lstruct1,lstruct2,0)

         isubthe=1
         call get_theory(isubthe,info)
         if(info.gt.0) goto 1000
         if(isubthe.gt.0) then
          iget=l2_get(lstruct2)
          if(aimag(array_fr(iget+1)).gt.1.0)then  
           b_phi=aimag(array_fr(iget+2)).gt.0.0
           phi  =real(array_fr(iget+2))
           theta=real(array_fr(iget+3))
           psi  =real(array_fr(iget+4))
          end if
         end if
!          write(6,'(2i6,a,3f10.2)')lstruct2,iget,            &
!           ' S mossbauer phi,theta,psi=',phi,theta,psi
          b_lay=(isubthe.gt.0).or.b_phi.or.b_thfactor.or.b_vfr2

          do lstruct3=istruct2(lstruct2-1)+1,istruct2(lstruct2)
          call mapsub(lstruct1,lstruct2,lstruct3)

          ! if one site of the source is polarized all sites
          ! need the full 2x2 matrices
          b_polisite(lstruct3)=c3_prp(lstruct3).eq.'plrs'
          if(b_polisite(lstruct3)) b_polsource=.true.

          if(th_factor.gt.0.) then
           wi(lstruct3)=vsstrf(2)*th_factor
          else
            wi(lstruct3)=vsstrf(2)
          end if

          isubthe=0
          call get_theory(isubthe,info)
          if(info.gt.0) goto 1000
          
          b_source=b_source.or.b_lay.or.(isubthe.gt.0).or.b_vsstrf(1)

          if(ifrcase.eq.6)then
           ! the density matrices of the sites of the source are summed up
           ! at the storage area of the first site
           isource=l3_get(lstruct3)
           nn=ipTHS+4*(2*nSC+1)+(lstruct3-1)*6+1
           icase=int(real(array_fr(nn)))
           array_fr(nn+5)=cmplx(1.,0.)
           if(icase.gt.1) then
            wixf=wi(lstruct3)*f_factor
            do i=-nSC,nSC
             na=4*(i+NSC)
             nn=isource+6*(i+NSC)
             scfunction(na+1)=scfunction(na+1)+array_fr(nn+3)*wixf
             scfunction(na+2)=scfunction(na+2)+array_fr(nn+4)*wixf
             scfunction(na+3)=scfunction(na+3)+array_fr(nn+5)*wixf
             scfunction(na+4)=scfunction(na+4)+array_fr(nn+6)*wixf
            end do
           end if
          end if
         end do  ! lstruct3
        end do   ! lstruct2
        if(ifrcase.eq.6)then
         if(b_source)then
          do i=-nSC,nSC
           na=ipTHS+4*(i+NSC)
           nn=4*(i+NSC)
           array_fr(na+1)=scfunction(nn+1)
           array_fr(na+2)=scfunction(nn+2)
           array_fr(na+3)=scfunction(nn+3)
           array_fr(na+4)=scfunction(nn+4)
          end do
         else
          do lstruct2=istruct1(lstruct1-1)+1,istruct1(lstruct1)
           do lstruct3=istruct2(lstruct2-1)+1,istruct2(lstruct2)
            nn=ipTHS+4*(2*nSC+1)+(lstruct3-1)*6+6
            array_fr(nn)=cmplx(0.,0.)
           end do
          end do
         end if
         deallocate(scfunction)
        end if
        b_device(1)=b_source
        b_total=b_total.or.b_source
       end if ! (nu_device(lstruct1)=1 source
       if(nu_device(lstruct1).eq.2) then
!       ----------------------------------------------------------------
!       layer lay2,3 ... mlay of  absorber at rest
!       ----------------------------------------------------------------
!       The Lorentz curve of the absorber in npq is 1 at resonance so that
!       for a single line the exponent at resonance is just -teff (effective
!       thickness). areaLor ~ pi*ga/2 ( area -+ infinity), ga/2=hga/2/sr
!       normalizes ( to area 1) the Lorentz curve.

!       Sums of absorber curves at the positions Eagd. For the convolution
!       integral the stepwidth is still srw4. The stepwidth srw8 of the
!       absorber function guarantees that the absorbtion line is exact
!       within 1/4 channel.
!
!       ----0--------------------------naLG
!       naLG--------------------------0-------------        srw8
!        nAS------------0------------nAS                    srw2
        lstr1case=2       ! device=absorber
        phi=0.; theta=0. ; psi=0.

        if(ifrcase.eq.5)then
         areafrac(lstruct1)=devfactor
         b_device(lstruct1)=b_devfactor
!        rotation of the absorber, the same for all layers
         call mapsub(lstruct1,0,0)
         isubthe=1
         call get_theory(isubthe,info)
         if(info.gt.0) goto 1000
         if(isubthe.eq.0) then
          b_phi=.false.
         else
          iget=l1_get(lstruct1)
          b_phi=aimag(array_fr(iget+2)).gt.0.0
          phi  =real(array_fr(iget+2))
          theta=real(array_fr(iget+3))
          psi  =real(array_fr(iget+4))
         end if
!         write(6,'(2i6,a,3f10.2)')lstruct1,iget,      &
!         '  mossbauer phi,theta,psi=',phi,theta,psi

!        the external field is the same for all layers of device2=absorb
!        for all mosaic parts of the absorber -> transfer of variables
!        and logical from common/uvfr1trf/ to common/uvfr2trf/ (u=a,b).
         Hext=devHext
         thetaHe=devthetaHe
         phiHe=devphiHe
         b_dumx(5)=b_devHext
         b_dumx(6)=b_devthetaHe
         b_dumx(7)=b_devphiHe
        end if

        do lstruct2=istruct1(lstruct1-1)+1,istruct1(lstruct1)
         ipostxtu=lstruct2
         b_lay=.false.
          call mapsub(lstruct1,lstruct2,0)

          isubthe=1
          call get_theory(isubthe,info)
!           write(6,'(a,3i6)')'isubthe,ifrcase=',isubthe,ifrcase

          if(info.gt.0) goto 1000
          if(ifrcase.ne.5)then
           if(isubthe.eq.0) then
            b_phi=.false.
           else
            iget=l2_get(lstruct2)
            if(aimag(array_fr(iget+1)).gt.1.0)then  
             b_phi=aimag(array_fr(iget+2)).gt.0.0
             phi  =real(array_fr(iget+2))
             theta=real(array_fr(iget+3))
             psi  =real(array_fr(iget+4))
            end if
           end if
           b_lay= b_lay.or.b_phi
          end if

         b_lay= b_lay.or.(isubthe.gt.0).or.b_vfr2
         do lstruct3=istruct2(lstruct2-1)+1,istruct2(lstruct2)
          b_polisite(lstruct3)=c3_prp(lstruct3).eq.'plrs'
          if(b_polisite(lstruct3)) b_polabsorber=.true.
          call mapsub(lstruct1,lstruct2,lstruct3)

          sftr=vsstrf(2)

!         logical of the thickness sftr is at b_vsstrf(2)
          if(ifrcase.eq.3.or.ifrcase.eq.5.or.ifrcase.eq.6) then
           factor_thickness=real(array_fr(iget+1))

           wi(lstruct3)=sftr*uthick_w*factor_thickness
           if(th_factor.gt.0.001) then
            wi(lstruct3)=wi(lstruct3)*th_factor
            b_lay=b_lay.or.b_thfactor.or.b_vsstrf(2)
           end if
          else
             if(b_reemission) then
              wi(lstruct3)=sftr
             else
              wi(lstruct3)=sftr*uthick_w
             end if
           if(th_factor.gt.0.001) then
            wi(lstruct3)=wi(lstruct3)*th_factor
            b_lay= b_lay.or.b_thfactor.or.b_vsstrf(2)
           end if
          end if
          isubthe=0
          call get_theory(isubthe,info)
          if(info.gt.0) goto 1000
!          write(6,'(a)')' th_mossbauer absorber 2'
!          write(6,'(10f10.3)')(vsstrf(i),i=1,10)
          b_lay= b_lay.or.b_vsstrf(2).or.(isubthe.gt.0)
         end do
          b_module(lstruct2)=b_lay
          b_device(lstruct1)=b_device(lstruct1).or.b_lay
        end do
       end if  !  nu_device(lstruct1)=2 absorber
       b_absorber=b_absorber.or.b_device(lstruct1)
       b_phi=.false.

       Arho=(0.,0.); b_pp=.true.
       if(nu_device(lstruct1).eq.3) then
!       hj=(hj-float(i))*dx_the;  hj1=(hj1-float(i))*dx_the; sv=hj+hj1; pv=hj1*hj
        lstr1case=3
!      ----------------------------------------------------------------
!      layer lay=mlay of detector
!      ----------------------------------------------------------------
        do n=-nSC,nSC
         detector(n)=0.
        end do

        b_absdetector=.true.
        do lstruct2=istruct1(lstruct1-1)+1,istruct1(lstruct1)

         lay=lstruct2
         do lstruct3=istruct2(lstruct2-1)+1,istruct2(lstruct2)


          call mapsub(lstruct1,lstruct2,lstruct3)
          j=int(vsstrf(1))
          do k=3,j+1
           b_not_sftr=b_not_sftr.or.b_vsstrf(k)
          end do

          sftr=vsstrf(2)
          wi(lstruct3)=sftr*uthick_w

          isubthe=0
          call get_theory(isubthe,info)
          if(info.gt.0) goto 1000

!          b_detector=b_not_sftr.or.b_vsstrf(2).or.(isubthe.gt.0)
          b_detector=(isubthe.gt.0)

!          print'(a,5l5)','b_not_sftr,b_vsstrf(2),(isubthe.gt.0)=',  &
!              b_not_sftr,b_vsstrf(2),(isubthe.gt.0)

          idt=l3_get(lstruct3)+1; n=idt+nSC
          !      Fortran  ( 1 3 )     convention
          !               ( 2 4 )
          Arho(1)=array_fr(n+1); Arho(2)=array_fr(n+2)
          Arho(3)=array_fr(n+3); Arho(4)=array_fr(n+4)
          b_pp=real(array_fr(n+5)).eq.0.0
          b_detector= b_detector.or.aimag(array_fr(n+5)).eq.1.0.or.b_pp

     if(enwr(7).eq.'3rho')then
      print'(a,4f8.2)', 'An  rho=', Arho(1),Arho(3)
      print'(a,4f8.2)', '        ', Arho(2),Arho(4)
      print'(a,2l5)','b_pp, b_detector=',b_pp, b_detector
     end if

          do n=0,nSC
           detector(n)=detector(n)+real(array_fr(idt+n))*wi(lstruct3)
           detector(-n)=detector(-n)+aimag(array_fr(idt+n))*wi(lstruct3)
          end do  ! n=0,nSC
          do n=-nSC,nSC
           detector(n)=exp(-2.*detector(n))
          end do  ! n=0,nSC
         end do
        end do
        b_device(lstruct1)=b_detector
       end if  !  nu_device(lstruct1)=3 detector
      end do
      b_total=b_total.or.b_source.or.b_absorber.or.b_detector

!-----------------------------------------------------------------------
!      here the matrix R for the absorber are given :
!      satrace(n)*1 +  (sapp(n)  sapm(n))
!                      (samp(n) -sapp(n))
!
!      calculation of the transmittivity matrix T for all channels n
!      by   T = exp(iR)*rho*exp(-iR+)
!      exp(-2*Sum;lay sattrlay(lay)) is a common factor, which is the
!      only one for an unpolarized sample

!-----------------------------------------------------------------------
!      here the matrix R for the layers are given :
!      satrlay(n,lay)*1 +  (sapplay(n,lay)  sapmlay(n,lay))
!                          (samplay(n,lay) -sapplay(n,lay))
!
!      calculation of the transmittivity matrix T for all channels n
!      by   T = exp(iR)*rho*exp(-iR+)
!      exp(-2*Sum;lay sattrilay(lay)) is a common factor, which is the
!      only one for an unpolarized sample
!      if(mstrut1.eq.3) then the last layer (detector) does not move rel
!      to the source. The detector window is treated as an unpolarized a

!       if(enwr(15).eq.'tota')
!       write(6,'(a,5l5)')'moss b_source,b_absorber,b_detector,b_total=',  &
!                            b_source,b_absorber,b_detector,b_total
      Arho=2.0*Arho
!     analyser at P=0 is the unit matrix

      if(.not.b_total) goto 1000
         call cpu_time(start); call wall_time(wstart)

      iyout=ipTH+1
      do k=0,2*km+1
       array_fr(iyout+k)=cmplx(0.,0.)
      end do

      mfr1=mstruct1
      if(nu_device(mstruct1).eq.3)mfr1=mfr1-1
      do lstruct1=2,mfr1

         if(b_device(lstruct1).or..not.b_pp) then

        do n=-nAS,nAS
         n2=2*abs(n)+1
         if(n.ge.0) n2=n2-1
         n8=4*n2

         satraceh=cmplx(0.,0.)
         do lay=istruct1(lstruct1-1)+1,istruct1(lstruct1)
          issa=istruct2(lay-1)+1
          isse=istruct2(lay)
          app=cmplx(0.,0.)
          apm=cmplx(0.,0.)
          amp=cmplx(0.,0.)
          if(isse.gt.issa) then
           do isite=issa,isse
!           if(n.eq.0)write(6,'(a ,2i4,2x,f10.4)')   &
!            ' --1-lay,isite,wi(isite)=',lay,isite,wi(isite)
            if(b_polisite(isite)) then
             nn=l3_get(isite)+n8
!            if(n.eq.0)write(6,'(a,2i6)') 'iget=',l3_get(isite),nAS
             app=app+array_fr(nn+2)*wi(isite)
             apm=apm+array_fr(nn+3)*wi(isite)
             amp=amp+array_fr(nn+4)*wi(isite)
            else
             nn=l3_get(isite)+n2
!             if(n.eq.0)write(6,'(a,i6)') 'iget=',l3_get(isite)
            end if
            satraceh=satraceh+array_fr(nn+1)*wi(isite)

!            if(abs(n).lt.2) write(6,'(2i6,11f8.3)')&
!             n,nn,wi(isite),satraceh*1000
                      ! isite loop
           end do
          else
!          if(n.eq.0)write(6,'(a,i6)') 'iget=',l3_get(issa)
!          if(n.eq.0)write(6,'(a ,2i4,2x,f10.4,l5)')  &
!                 ' --2-lay,issa,wi(issa)=',lay,issa,wi(issa),b_polisite(issa)

           if(b_polisite(issa)) then
            nn=l3_get(issa)+n8
            app   =  array_fr(nn+2)*wi(issa)
            apm   =  array_fr(nn+3)*wi(issa)
            amp   =  array_fr(nn+4)*wi(issa)
!           if(n.eq.0)write(6,'(a ,2i8,3i4,2x,f10.4)')  &
!          '1-lay,issa,wi(issa)=',l3_get(issa),l3_get(issa)+8*nAS, &
!           lstruct1,lay,issa,wi(issa)
           else
            nn=l3_get(issa)+n2
           end if
           satraceh=satraceh+array_fr(nn+1)*wi(issa)
!           if(n.eq.10)write(6,'(a,2i6,3f12.3)')&
!                'nn=',l3_get(issa),nn,wi(issa),satraceh*1000
          end if  ! isse.gt.issa

          if(b_polabsorber) then
           if(b_reemission) then
            aa(1)=cmplx(1.0,0.0)+app
            aa(3)=apm
            aa(2)=amp
            aa(4)=cmplx(1.0,0.0)-app
           else
            call expo_22(app,apm,amp,aa)
           end if
           do k=1,4
              aam(k,lay)=aa(k)
           end do
          end if
         end do   ! lay

         if(b_reemission) then
          satrexpr=1.0-2.*aimag(satraceh)
         else
          if(enwr(15).eq.'cten')then
           satrexpr=1.-aimag(satraceh)*2.
          else
           satrexpr=exp(-aimag(satraceh)*2.)
          end if
         end if
!         if(abs(n).le.5) write(6,'(i2,3f12.6,5x,L5)') &
!                 n,satraceh,satrexpr,b_polabsorber
         layl=istruct1(lstruct1)
         layb=istruct1(lstruct1-1)+1
         if(b_polabsorber) then
          do k=1,4
           dd(k)=aam(k,layl)
          end do

!         multiplication of the matrices exp(iR(layl))*...*exp(iR(2))
          if(layl.gt.2) then
           do k=1,4
            dcc(k)=dcmplx(dd(k))
           end do
           do lay=layl-1,layb,-1
            do k=1,4
             daa(k)=dcc(k)
             dbb(k)=dcmplx(aam(k,lay))
            end do
            call matproD(daa,dbb,dcc)
           end do
           do k=1,4
            dd(k)=dcc(k)
           end do
          end if
           do k=1,8
            if(abs(ri(k)).lt.1.e-9)ri(k)=0.0
           end do
!         if(n.gt.710) write(6,'(a,i5,8g12.3)')' dd=',n,dd
!         trace (dd+*dd*rho) is the intensity
!         bb=cc+
          if(.not.b_pp)then
            bb=dd
            call matprof(bb,Arho,dd)
          end if
          bb(1)=conjg(dd(1))
          bb(4)=conjg(dd(4))
          bb(2)=conjg(dd(3))
          bb(3)=conjg(dd(2))

          call matprof(bb,dd,aa)

!         aa=cc*cc+ is a Hermitian matrix
!         aa=1/2*trace(aa)*1 + aP*sv(Pauli)
          aa(1)=aa(1)*satrexpr
          aa(2)=aa(2)*satrexpr
          aa(4)=aa(4)*satrexpr
          aTr(n)= 0.5*real(aa(1)+aa(4)) -1.
          aP1(n)= real(aa(2))
          aP2(n)=-aimag(aa(2))
          aP3(n)=0.5*real(aa(1)-aa(4))
         else
          aTr(n)= satrexpr-1.
          aP1(n)=0.0
          aP2(n)=0.0
          aP3(n)=0.0
         end if
         eTr(n)=-satrexpr*aimag(satraceh)*2.
         end do ! n

        nabs0=l1_put(lstruct1)+1
        if(b_polabsorber.and.b_polsource) then
         nabs1=nabs0+nAS+1
         nabs2=nabs1+nAS+1
         nabs3=nabs2+nAS+1
         do n=0,nAS
          array_fr(nabs0+n)=cmplx(aTr(n),aTr(-n))
          array_fr(nabs1+n)=cmplx(aP1(n),aP1(-n))
          array_fr(nabs2+n)=cmplx(aP2(n),aP2(-n))
          array_fr(nabs3+n)=cmplx(aP3(n),aP3(-n))
         end do
      else
         do n=0,nAS
          array_fr(nabs0+n)=cmplx(aTr(n),aTr(-n))
         end do
        end if
       else  ! b_device(lstruct1)
        nabs0=l1_put(lstruct1)+1              ! 25.7.06
        if(b_polabsorber.and.b_polsource) then
         nabs1=nabs0+nAS+1
         nabs2=nabs1+nAS+1
         nabs3=nabs2+nAS+1
         do n=0,nAS
          aTr(n) = real(array_fr(nabs0+n))
          aP1(n) = real(array_fr(nabs1+n))
          aP2(n) = real(array_fr(nabs2+n))
          aP3(n) = real(array_fr(nabs3+n))
          aTr(-n)=aimag(array_fr(nabs0+n))
          aP1(-n)=aimag(array_fr(nabs1+n))
          aP2(-n)=aimag(array_fr(nabs2+n))
          aP3(-n)=aimag(array_fr(nabs3+n))
         end do
        else
         do n=0,nAS
          aTr(n) = real(array_fr(nabs0+n))
          aTr(-n)=aimag(array_fr(nabs0+n))
         end do
        end if
       end if       ! b_device(lstruct1)


       nspdv=l1_put(lstruct1)+4*(nAS+1)+1
!!     array_fr(nspdv+k)=(1.,1.)   
       do k=0,km
        array_fr(nspdv+k)=(1.,1.)
       end do

      do lstruct2=istruct1(0)+1,istruct1(1)
       lay= lstruct2
       issa=istruct2(lay-1)+1
       isse=istruct2(lay)

      do isite=issa,isse
!      sum over all source sites
       isource=l3_get(isite)
!      write(6,'(a,f10.2,2i6)') 'wi(isite) isite,isource=',wi(isite),isite,isource
       factor_thickness=wi(isite)
       b_polsource=b_polisite(isite)
      if(factor_thickness.gt.0.)then
        if(ifrcase.eq.6)then
         if(enwr(7).eq.'scrv')then
          call rhoz_reverse(isource,isite,ipTHS,nSC,mAS,sTr,sP1,sP2,sP3,attenuation)
         else
          call rhoz(isource,isite,ipTHS,nSC,mAS,sTr,sP1,sP2,sP3,attenuation)
         end if
         nn=ipTHS+4*(2*nSC+1)+6*(isite-1)+1
         icase=int(real(array_fr(nn)))
         if(icase.gt.1) then
          hnorm=0.
          do n=-nSC+1,nSC-1,2
             hnorm=hnorm+ sTr(n)*2.0
          end do
          do n=-nSC+2,nSC-2,2
           hnorm=hnorm+sTr(n)
          end do
          hnorm=(hnorm*2.+ sTr(-nSC)+sTr(nSC))/3.
          ff=f_factor*hnorm+(1.-f_factor)*attenuation
          anorm(isite)=ff
          f_eff=hnorm/ff*f_factor

          ff=hnorm
          do n=-nSC,nSC
           sTr(n)=sTr(n)/ff
           sP1(n)=sP1(n)/ff
           sP2(n)=sP2(n)/ff
           sP3(n)=sP3(n)/ff
          end do

          if(enwr(15).eq.'scef') then
            print'(a,i2,2(a,f6.3),a,f6.3/,a,f6.3)', &
                 'site',isite,' Ir=',f_factor*hnorm,&
            ' Inr=',(1.0-f_factor)*attenuation,' f_eff=',f_eff, &
              ' bg_eff=1-f_factor/f_eff*(1.-bg_fraction)=',  &
              1-f_factor/f_eff*(1.-bg_fraction)
           querytype='interpretation of variables Ir,Inr,f_eff'
           chcom='<Mossbauer_spectrum>/attenuation'
           ch1='n'
           nn=80
           call yesno(querytype,ch1,chcom,nn,nrd,nwr)
           if(ch1.eq.'y')call help(chcom)
!           enwr(15)='nnnn'
          end if
         end if
        else
         if(b_polisite(isite)) then
          do n=0,nSC
           nn=isource+4*n
           sTr(n)=aimag(array_fr(nn+1))
           sP1(n)= real(array_fr(nn+2))
           sP2(n)=-aimag(array_fr(nn+2))
           sP3(n)= real(array_fr(nn+1))
           sTr(-n)=aimag(array_fr(nn+3))
           sP1(-n)= real(array_fr(nn+4))
           sP2(-n)=-aimag(array_fr(nn+4))
           sP3(-n)= real(array_fr(nn+3))
          end do
         else
          do n=0,nSC
           nn=isource+2*n
           sTr(n) =aimag(array_fr(nn+1))
           sTr(-n)=aimag(array_fr(nn+2))
          end do
          if(b_polabsorber)then
           sP1=0.;sP2=0.;sP3=0.
          end if
         end if
!        write(6,'(a,2i8)')'mossbau nSC,isource=',nSC,isource
!        write(12,'(i6,2x,f12.5)') (n,sTr(n)*100.,n=-nSC,nSC,16)
        end if

        if(b_absdetector) then
!        ----------------------------------------------------------
!        b_absdetector=.true.  detector at rest relative to the source
!        -----------------------------------------------------------
        ! holds if the detector scattering amplitude is scalar
        ! in case of no site (detector=0)/site=1 but PA neq 0,
        ! energy independent analyser: detector(n)=1.0
         h=SUM(detector)
         if(h.gt.0.1.and.h.lt.float(2*NSC)) then
          do n=-nSC,nSC
           sTr(n)=sTr(n)*detector(n)
           sP1(n)=sP1(n)*detector(n)
           sP2(n)=sP2(n)*detector(n)
           sP3(n)=sP3(n)*detector(n)
          end do
         end if
        end if

!       write(6,'(a,4i5)') ' npolSCAS=',npolSCAS,nAS, nSC
        npolSCAS=0
        if (b_polsource.and.b_polabsorber) npolSCAS=1
        if(enwr(14).eq.'cten')then
           do k=0,km
              yout(k)=cmplx(aTr(k*2),aTr(-k*2))
           end do
        else
         if(nAS.gt.nSC) then
          call convolutionAS(km,nSC,npolSCAS,yout)
         else
          call convolutionSC(km,nAS,npolSCAS,yout)
         end if
         call cpu_time(finish); call wall_time(wfinish)
        end if
!       write(6,'(a,f10.4,i4)')'thickness,isite=',factor_thickness,isite
        do k=0,km
         array_fr(nspdv+k)=array_fr(nspdv+k)+yout(k)*factor_thickness
        end do
        if(om_samp.gt.0) then
           aTr=eTr
           aP1=0.0
           aP2=0.0
           aP3=0.0
          if(nAS.gt.nSC) then  
             call convolutionAS(km,nSC,npolSCAS,yout)
         else
          call convolutionSC(km,nAS,npolSCAS,yout)
         end if
         do k=0,km
         n=iyout+km+1+k   
         array_fr(n)=array_fr(n)+yout(k)*factor_thickness
         end do
        end if ! om_samp.gt.0
        
       end if    ! factor_thickness.gt.0)
      end do     ! isite source loop
      end do     ! ilay  source loop
     end do      ! incoherent fraction
!    store yout(k)*factor_thickness for each absorber device (lstruct1)

      nspdv=l1_put(2)+4*(nAS+1)+1
      if(mfr1.gt.2)then
       if(ifrcase.eq.6)then
        do lstruct1=2,mfr1
         areafrac(lstruct1)=1.0
        end do
       else
        h=0.
        do lstruct1=2,mfr1
         h=h+areafrac(lstruct1)
        end do
        do lstruct1=2,mfr1
         areafrac(lstruct1)=areafrac(lstruct1)/h
        end do
       end if

       do lstruct1=2,mfr1
        nspdv=l1_put(lstruct1)+4*(nAS+1)+1
        do k=0,km
         array_fr(iyout+k)=array_fr(iyout+k)+                           &
                          array_fr(nspdv+k)*areafrac(lstruct1)
        end do
       end do     ! mfr1
      else
       do k=0,km
        array_fr(iyout+k)=array_fr(iyout+k)+array_fr(nspdv+k)
       end do
!       write(6,'(6g12.5)')(array_fr(iyout+k),k=0,km,16)
    
      end if
!-------------------------------------------------------------
!     end of calculation of absorber function
!     print '("#cpu/wall time = ",2f15.4," seconds.")',  &
!                            finish-start,wfinish-wstart

 1000 continue
      deallocate(detector)

      return
      END


     subroutine convolutionAS_(km,nSC,npolSCAS,yout)
     use glbconst, only: maxn,maxscrd
     implicit none

     integer km,nSC,npolSCAS
     integer,parameter::mAS=4*maxn,idumscrd=maxscrd-10*(2*mAS+1)
     real        sTr,sP1,sP2,sP3,aTr,aP1,aP2,aP3,eTr,sa,dumscrd
     common/scrd/sTr(-mAS:mAS),sP1(-mAS:mAS),sP2(-mAS:mAS),sP3(-mAS:mAS),&
                 aTr(-mAS:mAS),aP1(-mAS:mAS),aP2(-mAS:mAS),aP3(-mAS:mAS),&
                 eTr(-mAS:mAS),sa(-mAS:mAS),dumscrd(idumscrd)

     complex   yout(0:512)
     real      h1,h2
     integer   i,j,k
!-----------------------------------------------------------------------
!      convolution with the absorber function
!
!     -nAS-------k2--------------0---------k----k2--------nAS
!     -nSC---k2--0--k2---nSC  -nSC---k2--0--k2---nSC
!

!     this convolution is calculated with stepwidth srw2
!         write(6,'(a,i8)') '  -conAS--, nSC=',nSC

!       do j=-nSC/8,nSC/8
!        write(6,'(i4,8e10.3)') j,(aTr((j-1)*8+i),i=1,8)
!       end do
!       write(6,'(a,2i6)') 'convolutionAS kzlm,nSC=',kzlm,nSC
!       write(6,'(a,i8)') 'convolutionAS npolSCAS=',npolSCAS
!       write(6,'(4f10.3)') (sTR(i),i=-2,1)
!       write(6,'(i4,a,2f15.6)')  &
!         (k,' sTr(k),aTr(k)=',sTr(k),aTr(k),k=-1,1)

      do i=0,2*km,2
       h1=0.
       h2=0.
        do j=-nSC+1,nSC-1,2
        h1=h1+ sTr(j)*aTr(i+j)*2.
        h2=h2+ sTr(j)*aTr(-i+j)*2.
        end do
        do j=-nSC+2,nSC-2,2
        h1=h1+ sTr(j)*aTr(i+j)
        h2=h2+ sTr(j)*aTr(-i+j)
        end do
        h1=h1*2.+ sTr(-nSC)*aTr(-nSC+i)+sTr(nSC)*aTr(nSC+i)
        h2=h2*2.+ sTr(-nSC)*aTr(-nSC-i)+sTr(nSC)*aTr(nSC-i)

      if (npolSCAS.eq.1) then
!     product   sP * aP  , Poincare vector of the source sP is already
!     weighted by the intensity sTr
        do j=-nSC+1,nSC-1,2
        h1=h1+ (sP1(j)*aP1(i+j)+sP2(j)*aP2(i+j)+sP3(j)*aP3(i+j))*4.
        h2=h2+ (sP1(j)*aP1(-i+j)+sP2(j)*aP2(-i+j)+sP3(j)*aP3(-i+j))*4.
        end do
        do j=-nSC+2,nSC-2,2
        h1=h1+ (sP1(j)*aP1(i+j)+sP2(j)*aP2(i+j)+sP3(j)*aP3(i+j))*2.
        h2=h2+ (sP1(j)*aP1(-i+j)+sP2(j)*aP2(-i+j)+sP3(j)*aP3(-i+j))*2.
        end do
        k=-nSC+i
        h1=h1+ sP1(-nSC)*aP1(k)+sP2(-nSC)*aP2(k)+sP3(-nSC)*aP3(k)
        k=nSC+i
        h1=h1+ sP1(nSC)*aP1(k)+sP2(nSC)*aP2(k)+sP3(nSC)*aP3(k)
        k=-nSC-i
        h2=h2+ sP1(-nSC)*aP1(k)+sP2(-nSC)*aP2(k)+sP3(-nSC)*aP3(k)
        k=nSC-i
        h2=h2+ sP1(nSC)*aP1(k)+sP2(nSC)*aP2(k)+sP3(nSC)*aP3(k)

       end if     ! if polarized- condition
       yout(i/2)=cmplx(h1,h2)/3. !-cmplx(1.,1.)
      end do
      return
      END

     subroutine convolutionSC(km,nAS,npolSCAS,yout)
     use glbconst, only: maxn,maxscrd
     implicit none

     integer km,nAS,npolSCAS
     integer,parameter::mAS=4*maxn,idumscrd=maxscrd-10*(2*mAS+1)
     real        sTr,sP1,sP2,sP3,aTr,aP1,aP2,aP3,eTr,sa,dumscrd
     common/scrd/sTr(-mAS:mAS),sP1(-mAS:mAS),sP2(-mAS:mAS),sP3(-mAS:mAS),&
                 aTr(-mAS:mAS),aP1(-mAS:mAS),aP2(-mAS:mAS),aP3(-mAS:mAS),&
                 eTr(-mAS:mAS),sa(-mAS:mAS),dumscrd(idumscrd)

     complex   yout(0:512)
     real    h1,h2,hi,hr
     integer i,j,k
!    write(6,'(a,i4)') 'convolutionSC sTR,aTR,npolSCAS=',npolSCAS
!    write(6,'(a,2i8)') 'convolutionSC nAS=',nAS
!    write(11,'(i5,2f15.5)')(i,sTr(i),aTr(i),i=-nAS,nAS)

      do i=0,2*km,2
       do j=-nAS,nAS
        sa(j)=aTr(j)*sTr(i+j)
       end do
       h1=0.
       do j=-nAS+1,nAS-1,2
        h1=h1+ sa(j)*4.
       end do
       do j=-nAS+2,nAS-2,2
        h1=h1+ sa(j)*2
       end do
       h1=h1+ sa(-nAS)+sa(nAS)
       h2=0.
       do j=-nAS+3,nAS-5,2
        h2=h2+ sa(j)*8.
       end do
       do j=-nAS+2,nAS-4,2
        h2=h2- sa(j)*8.
       end do
       h2=h2-4.*(sa(-nAS)+sa(nAS-2))                                    &
     &      +7.*(sa(-nAS+1)+sa(nAS-3))
       hr=h1-h2/30.

       do j=-nAS,nAS
        sa(j)=aTr(j)*sTr(-i+j)
       end do
       h1=0.
       do j=-nAS+1,nAS-1,2
        h1=h1+ sa(j)*4.
       end do
       do j=-nAS+2,nAS-2,2
        h1=h1+ sa(j)*2
       end do
       h1=h1+ sa(-nAS)+sa(nAS)
       h2=0.
       do j=-nAS+3,nAS-5,2
        h2=h2+ sa(j)*8.
       end do
       do j=-nAS+2,nAS-4,2
        h2=h2- sa(j)*8.
       end do
       h2=h2-4.*(sa(-nAS)+sa(nAS-2))                                    &
            +7.*(sa(-nAS+1)+sa(nAS-3))
       hi=h1-h2/30.

       if (npolSCAS.eq.1) then
!       product   aP * sP  , Poincare vector of the source sP is already
!       weighted by the intensity sTr
        do j=-nAS,nAS
         sa(j)=aP1(j)*sP1(i+j)+aP2(j)*sP2(i+j)+aP3(j)*sP3(i+j)
        end do
        h1=0.
        do j=-nAS+1,nAS-1,2
         h1=h1+ sa(j)*4.
        end do
        do j=-nAS+2,nAS-2,2
         h1=h1+ sa(j)*2
        end do
        h1=h1+ sa(-nAS)+sa(nAS)
        h2=0.
        do j=-nAS+3,nAS-5,2
         h2=h2+ sa(j)*8.
        end do
        do j=-nAS+2,nAS-4,2
         h2=h2- sa(j)*8.
        end do
        h2=h2-4.*(sa(-nAS)+sa(nAS-2))                                   &
             +7.*(sa(-nAS+1)+sa(nAS-3))
        hr=hr+h1-h2/30.

        do j=-nAS,nAS
         sa(j)=aP1(j)*sP1(-i+j)+aP2(j)*sP2(-i+j)+aP3(j)*sP3(-i+j)
        end do
        h1=0.
        do j=-nAS+1,nAS-1,2
         h1=h1+ sa(j)*4.
        end do
        do j=-nAS+2,nAS-2,2
         h1=h1+ sa(j)*2
        end do
        h1=h1+ sa(-nAS)+sa(nAS)
        h2=0.
        do j=-nAS+3,nAS-5,2
         h2=h2+ sa(j)*8.
        end do
        do j=-nAS+2,nAS-4,2
         h2=h2- sa(j)*8.
        end do
        h2=h2-4.*(sa(-nAS)+sa(nAS-2))                                   &
             +7.*(sa(-nAS+1)+sa(nAS-3))
        hi=hi+h1-h2/30.
       end if     ! if polarized- condition
       k=i/2
       yout(k)=cmplx(hi,hr)/3. !-cmplx(1.,1.)
      end do
      return
      END

     subroutine convolutionAS(km,nSC,npolSCAS,yout)
     use glbconst, only: maxn,maxscrd
     implicit none

     integer km,nSC,npolSCAS
     integer,parameter::mAS=4*maxn,idumscrd=maxscrd-10*(2*mAS+1)
     real        sTr,sP1,sP2,sP3,aTr,aP1,aP2,aP3,eTr,sa,dumscrd
     common/scrd/sTr(-mAS:mAS),sP1(-mAS:mAS),sP2(-mAS:mAS),sP3(-mAS:mAS),&
                 aTr(-mAS:mAS),aP1(-mAS:mAS),aP2(-mAS:mAS),aP3(-mAS:mAS),&
                 eTr(-mAS:mAS),sa(-mAS:mAS),dumscrd(idumscrd)

     complex   yout(0:512)
     real    h1,h2,hi,hr
     integer i,j,k
!-----------------------------------------------------------------------
!      convolution with the absorber function
!
!     -nAS-------k2--------------0---------k----k2--------nAS
!     -nSC---k2--0--k2---nSC  -nSC---k2--0--k2---nSC
!

!     this convolution is calculated with stepwidth srw2
!         write(6,'(a,i8)') '  -conAS--, iscn=',iscn

!       do j=-nSC/8,nSC/8
!        write(6,'(i4,8e10.3)') j,(aTr((j-1)*8+i),i=1,8)
!       end do
!       write(6,'(a,2i6)') 'convolutionAS km,nSC=',km,nSC
!       write(6,'(a,i8)') 'convolutionAS npolSCAS=',npolSCAS
!      write(11,'(i5,2f15.5)')(i,sTr(i),aTr(i),i=-nSC,nSC)


     do i=0,2*km,2

       do j=-nSC,nSC
        sa(j)=sTr(j)*aTr(i+j)
       end do
       h1=0.
       do j=-nSC+1,nSC-1,2
        h1=h1+ sa(j)*4.
       end do
       do j=-nSC+2,nSC-2,2
        h1=h1+ sa(j)*2
       end do
       h1=h1+ sa(-nSC)+sa(nSC)
       h2=0.
       do j=-nSC+3,nSC-5,2
        h2=h2+ sa(j)*8.
       end do
       do j=-nSC+2,nSC-4,2
        h2=h2- sa(j)*8.
       end do
       h2=h2-4.*(sa(-nSC)+sa(nSC-2))                                    &
            +7.*(sa(-nSC+1)+sa(nSC-3))
       hr=h1-h2/30.

       do j=-nSC,nSC
        sa(j)=sTr(j)*aTr(-i+j)
       end do
       h1=0.
       do j=-nSC+1,nSC-1,2
        h1=h1+ sa(j)*4.
       end do
       do j=-nSC+2,nSC-2,2
        h1=h1+ sa(j)*2
       end do
       h1=h1+ sa(-nSC)+sa(nSC)
       h2=0.
       do j=-nSC+3,nSC-5,2
        h2=h2+ sa(j)*8.
       end do
       do j=-nSC+2,nSC-4,2
        h2=h2- sa(j)*8.
       end do
       h2=h2-4.*(sa(-nSC)+sa(nSC-2))                                    &
            +7.*(sa(-nSC+1)+sa(nSC-3))
       hi=h1-h2/30.

       if (npolSCAS.eq.1) then
!      product   sP * aP  , Poincare vector of the source sP is already
!      weighted by the intensity sTr
        do j=-nSC,nSC
         sa(j)=sP1(j)*aP1(i+j)+sP2(j)*aP2(i+j)+sP3(j)*aP3(i+j)
        end do
        h1=0.
        do j=-nSC+1,nSC-1,2
         h1=h1+ sa(j)*4.
        end do
        do j=-nSC+2,nSC-2,2
         h1=h1+ sa(j)*2
        end do
        h1=h1+ sa(-nSC)+sa(nSC)
        h2=0.
        do j=-nSC+3,nSC-5,2
         h2=h2+ sa(j)*8.
        end do
        do j=-nSC+2,nSC-4,2
         h2=h2- sa(j)*8.
        end do
        h2=h2-4.*(sa(-nSC)+sa(nSC-2))                                   &
             +7.*(sa(-nSC+1)+sa(nSC-3))
        hr=hr+h1-h2/30.

        do j=-nSC,nSC
         sa(j)=sP1(j)*aP1(-i+j)+sP2(j)*aP2(-i+j)+sP3(j)*aP3(-i+j)
        end do
        h1=0.
        do j=-nSC+1,nSC-1,2
         h1=h1+ sa(j)*4.
        end do
        do j=-nSC+2,nSC-2,2
         h1=h1+ sa(j)*2
        end do
        h1=h1+ sa(-nSC)+sa(nSC)
        h2=0.
        do j=-nSC+3,nSC-5,2
         h2=h2+ sa(j)*8.
        end do
        do j=-nSC+2,nSC-4,2
         h2=h2- sa(j)*8.
        end do
        h2=h2-4.*(sa(-nSC)+sa(nSC-2))                                   &
             +7.*(sa(-nSC+1)+sa(nSC-3))
        hi=hi+h1-h2/30.
       end if     ! if polarized- condition
       k=i/2
       yout(k)=cmplx(hr,hi)/3.   !-cmplx(1.,1.)
      end do
      return
      END


